//////////////////////////////////////////////////////////////////////////////
// * File name: dma.c
// *                                                                          
// * Description:  DMA Functions.
// *                                                                          
// * Copyright (C) 2010 Texas Instruments Incorporated - http://www.ti.com/ 
// * Copyright (C) 2010 Spectrum Digital, Incorporated
// *                                                                          
// *                                                                          
// *  Redistribution and use in source and binary forms, with or without      
// *  modification, are permitted provided that the following conditions      
// *  are met:                                                                
// *                                                                          
// *    Redistributions of source code must retain the above copyright        
// *    notice, this list of conditions and the following disclaimer.         
// *                                                                          
// *    Redistributions in binary form must reproduce the above copyright     
// *    notice, this list of conditions and the following disclaimer in the   
// *    documentation and/or other materials provided with the                
// *    distribution.                                                         
// *                                                                          
// *    Neither the name of Texas Instruments Incorporated nor the names of   
// *    its contributors may be used to endorse or promote products derived   
// *    from this software without specific prior written permission.         
// *                                                                          
// *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS     
// *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT       
// *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR   
// *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT    
// *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,   
// *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT        
// *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,   
// *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY   
// *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT     
// *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   
// *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    
// *                                                                          
//////////////////////////////////////////////////////////////////////////////
 
#include "usbstk5505.h"
#include "csl_dma.h"

#define XMIT_BUFF_SIZE  48

/* -3dB 1 KHz sine sampled at 48 KHz */
Int16 XmitL_Sine_1K[XMIT_BUFF_SIZE] = {
0x0000,0x0B6C,0x16A6,0x217C,0x2BC0,0x3544,0x3DDF,0x456B, 
0x4BC7,0x50D7,0x5485,0x56C0,0x5780,0x56C0,0x5485,0x50D7, 
0x4BC7,0x456B,0x3DDF,0x3544,0x2BC0,0x217C,0x16A6,0x0B6C, 
0x0000,0xF494,0xE95A,0xDE84,0xD440,0xCABC,0xC221,0xBA95, 
0xB439,0xAF29,0xAB7B,0xA940,0xA880,0xA940,0xAB7B,0xAF29, 
0xB439,0xBA95,0xC221,0xCABC,0xD440,0xDE84,0xE95A,0xF494
};

/* -3 dB 2 KHz sine sampled at 48 KHz */
Int16 XmitL_Sine_2K[XMIT_BUFF_SIZE] = {
0x0000,0x16A6,0x2BC0,0x3DDF,0x4BC7,0x5485,0x5780,0x5485, 
0x4BC7,0x3DDF,0x2BC0,0x16A6,0x0000,0xE95A,0xD440,0xC221, 
0xB439,0xAB7B,0xA880,0xAB7B,0xB439,0xC221,0xD440,0xE95A,
0x0000,0x16A6,0x2BC0,0x3DDF,0x4BC7,0x5485,0x5780,0x5485, 
0x4BC7,0x3DDF,0x2BC0,0x16A6,0x0000,0xE95A,0xD440,0xC221, 
0xB439,0xAB7B,0xA880,0xAB7B,0xB439,0xC221,0xD440,0xE95A
};

/* -3dB 1 KHz sine sampled at 48 KHz */
Int16 XmitR_Sine_1K[XMIT_BUFF_SIZE] = {
0x0000,0x0B6C,0x16A6,0x217C,0x2BC0,0x3544,0x3DDF,0x456B, 
0x4BC7,0x50D7,0x5485,0x56C0,0x5780,0x56C0,0x5485,0x50D7, 
0x4BC7,0x456B,0x3DDF,0x3544,0x2BC0,0x217C,0x16A6,0x0B6C, 
0x0000,0xF494,0xE95A,0xDE84,0xD440,0xCABC,0xC221,0xBA95, 
0xB439,0xAF29,0xAB7B,0xA940,0xA880,0xA940,0xAB7B,0xAF29, 
0xB439,0xBA95,0xC221,0xCABC,0xD440,0xDE84,0xE95A,0xF494
};

/* -3 dB 2 KHz sine sampled at 48 KHz */
Int16 XmitR_Sine_2K[XMIT_BUFF_SIZE] = {
0x0000,0x16A6,0x2BC0,0x3DDF,0x4BC7,0x5485,0x5780,0x5485, 
0x4BC7,0x3DDF,0x2BC0,0x16A6,0x0000,0xE95A,0xD440,0xC221, 
0xB439,0xAB7B,0xA880,0xAB7B,0xB439,0xC221,0xD440,0xE95A,
0x0000,0x16A6,0x2BC0,0x3DDF,0x4BC7,0x5485,0x5780,0x5485, 
0x4BC7,0x3DDF,0x2BC0,0x16A6,0x0000,0xE95A,0xD440,0xC221, 
0xB439,0xAB7B,0xA880,0xAB7B,0xB439,0xC221,0xD440,0xE95A
};

CSL_DMA_Handle      dmaHandle_L;
CSL_DMA_Handle      dmaHandle_R;
CSL_DMA_ChannelObj  dmaObj_L;
CSL_DMA_ChannelObj  dmaObj_R;

Int16 RcvL_Sine[XMIT_BUFF_SIZE];
Int16 RcvR_Sine[XMIT_BUFF_SIZE];
Uint16 DMA_Count =0, Error_Count_L =0, Error_Count_R =0;
Uint16 DMA_Done = 0;
extern Uint16 fSineWave;

/*
 *  set_dma0_ch0_i2s0_Lout( )
 *      Setup DMA channel 0 to send sine wave samples for I2S left data
 */
Uint16 set_dma0_ch0_i2s0_Lout(void)
{
    CSL_Status          status;
    Uint16              chanNumber;
    CSL_DMA_Config      dmaConfig;

    dmaConfig.pingPongMode = CSL_DMA_PING_PONG_DISABLE;


    dmaConfig.autoMode     = CSL_DMA_AUTORELOAD_ENABLE;
    dmaConfig.burstLen     = CSL_DMA_TXBURST_1WORD;
    dmaConfig.trigger      = CSL_DMA_EVENT_TRIGGER;
    dmaConfig.dmaEvt       = CSL_DMA_EVT_I2S0_TX;
    dmaConfig.dmaInt       = CSL_DMA_INTERRUPT_ENABLE;
    dmaConfig.chanDir      = CSL_DMA_WRITE;
    dmaConfig.trfType      = CSL_DMA_TRANSFER_IO_MEMORY;
    dmaConfig.dataLen      = XMIT_BUFF_SIZE*2;
    if(fSineWave ==1)  // Choose sine wave
        dmaConfig.srcAddr  = (Uint32)XmitL_Sine_1K;   
    else
        dmaConfig.srcAddr  = (Uint32)XmitL_Sine_2K;
    dmaConfig.destAddr     = 0x2a08;
    
    /* Setup channel 0 */
    status = DMA_init();
    chanNumber = 0;
    dmaHandle_L = DMA_open((CSL_DMAChanNum)chanNumber,&dmaObj_L, &status);
    status = DMA_config(dmaHandle_L, &dmaConfig);
    
    /* Start channel 1 */
    status = DMA_start(dmaHandle_L);

    return 0;
}

/*
 *  set_dma0_ch1_i2s0_Rout( )
 *      Setup DMA channel 1 to send sine wave samples for I2S right data
 */
Uint16 set_dma0_ch1_i2s0_Rout(void)
{   
    CSL_Status          status;
    Uint16              chanNumber;
    CSL_DMA_Config      dmaConfig;

    dmaConfig.pingPongMode = CSL_DMA_PING_PONG_DISABLE;
    dmaConfig.autoMode     = CSL_DMA_AUTORELOAD_ENABLE;
    dmaConfig.burstLen     = CSL_DMA_TXBURST_1WORD;
    dmaConfig.trigger      = CSL_DMA_EVENT_TRIGGER;
    dmaConfig.dmaEvt       = CSL_DMA_EVT_I2S0_TX;
    dmaConfig.dmaInt       = CSL_DMA_INTERRUPT_ENABLE;
    dmaConfig.chanDir      = CSL_DMA_WRITE;
    dmaConfig.trfType      = CSL_DMA_TRANSFER_IO_MEMORY;
    dmaConfig.dataLen      = XMIT_BUFF_SIZE*2;
    if(fSineWave ==1)  // Choose sine wave
        dmaConfig.srcAddr  = (Uint32)XmitR_Sine_1K;        
    else
        dmaConfig.srcAddr  = (Uint32)XmitR_Sine_2K;        
    dmaConfig.destAddr     = 0x2a0C;
    
    /* Setup channel 1 */
    status = DMA_init();
    chanNumber = 1;
    dmaHandle_R = DMA_open((CSL_DMAChanNum)chanNumber,&dmaObj_R, &status);
    status = DMA_config(dmaHandle_R, &dmaConfig);
    
    /* Start channel 1 */
    status = DMA_start(dmaHandle_R);

    return 0;
}

/*
 *  set_dma0_ch0_stop( )
 *      Stop DMA channel 0
 */
void set_dma0_ch0_stop(void)
{
    DMA_stop(dmaHandle_L); 
}

/*
 *  set_dma0_ch1_stop( )
 *      Stop DMA channel 1
 */
void set_dma0_ch1_stop(void)
{
    DMA_stop(dmaHandle_R);
}
