//////////////////////////////////////////////////////////////////////////////
// * File name: rtc.c
// *                                                                          
// * Description:  RTC functions.
// *                                                                          
// * Copyright (C) 2010 Texas Instruments Incorporated - http://www.ti.com/ 
// * Copyright (C) 2010 Spectrum Digital, Incorporated
// *                                                                          
// *                                                                          
// *  Redistribution and use in source and binary forms, with or without      
// *  modification, are permitted provided that the following conditions      
// *  are met:                                                                
// *                                                                          
// *    Redistributions of source code must retain the above copyright        
// *    notice, this list of conditions and the following disclaimer.         
// *                                                                          
// *    Redistributions in binary form must reproduce the above copyright     
// *    notice, this list of conditions and the following disclaimer in the   
// *    documentation and/or other materials provided with the                
// *    distribution.                                                         
// *                                                                          
// *    Neither the name of Texas Instruments Incorporated nor the names of   
// *    its contributors may be used to endorse or promote products derived   
// *    from this software without specific prior written permission.         
// *                                                                          
// *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS     
// *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT       
// *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR   
// *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT    
// *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,   
// *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT        
// *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,   
// *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY   
// *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT     
// *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE   
// *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.    
// *                                                                          
//////////////////////////////////////////////////////////////////////////////
 
#include "csl_rtc.h"
#include "stdio.h"
#include "usbstk5505.h"

Uint16 Flag_RTC=0;
Uint16 Count_RTC =0;

/*
 *  enable_rtc_second_int( )
 *      Setup RTC and enable interrupt
 */
void enable_rtc_second_int(void)
{
    RTC_eventEnable(CSL_RTC_SECEVENT_INTERRUPT);
    RTC_start();
}

/*
 *  reset_RTC(void)
 *      Reset RTC to initial values
 */
void reset_RTC(void)
{
    CSL_RtcTime      InitTime;
    CSL_RtcDate      InitDate;
    CSL_Status    status;
    
    InitDate.year  = 8;
    InitDate.month = 10;
    InitDate.day   = 16;

    InitTime.hours = 12;
    InitTime.mins  = 12;
    InitTime.secs  = 12;
    InitTime.mSecs = 12;
    
    status = RTC_setDate(&InitDate);
    status = RTC_setTime(&InitTime);
}

/*
 *  RTC_Isr(void)
 *      RTC interrupt service routine. This function is called
 *      whenever there is an RTC interrupt.
 */
interrupt void RTC_Isr(void)
{
    Uint16 temp;

    CSL_CPU_REGS->IFR1 = 0x0004;  // Clear RTC int flag
    
    temp = CSL_RTC_REGS->RTCINTFL;
    CSL_RTC_REGS->RTCINTFL = temp;

    Flag_RTC =1;
    Count_RTC ++;
}
