/*
 *  Copyright 2009 by Spectrum Digital Incorporated.
 *  All rights reserved. Property of Spectrum Digital Incorporated.
 */
// IWĩx[X́Ahmain_20200721_BeforeAddSynth_BUP.ch
//#############################################################################
// Orgel Resonater by T.Okutsu 20200407
//#############################################################################


#include "usbstk5505.h"
#include "usbstk5505_gpio.h"
#include "usbstk5505_i2c.h"
#include "stdio.h"
#include "register_system.h"//20140525 T.Okutsu for PLL_98MHz()


#define XmitL 0x10
#define XmitR 0x20


#define AIC3204_I2C_ADDR 0x18

#define IER0        *(volatile unsigned *)0x0000
#define IFR0        *(volatile unsigned *)0x0001
#define ST0_55      *(volatile unsigned *)0x0002
#define ST1_55      *(volatile unsigned *)0x0003
#define ST2_55      *(volatile unsigned *)0x004B
#define ST3_55      *(volatile unsigned *)0x0004
#define IER1        *(volatile unsigned *)0x0045
#define IFR1        *(volatile unsigned *)0x0046

//**************************************************************************
//        DMA0 Ch0 Register Addresses
//**************************************************************************
#define DMA0_CH0_SRC_LSW        *(ioport volatile unsigned *)0x0c00    // DMA0 Channel 0 Source LSW Address
#define DMA0_CH0_SRC_MSW        *(ioport volatile unsigned *)0x0c01    // DMA0 Channel 0 Source MSW Address
#define DMA0_CH0_DST_LSW        *(ioport volatile unsigned *)0x0c02    // DMA0 Channel 0 Destination LSW Address
#define DMA0_CH0_DST_MSW        *(ioport volatile unsigned *)0x0c03    // DMA0 Channel 0 Destination MSW Address
#define DMA0_CH0_TC_LSW         *(ioport volatile unsigned *)0x0c04    // DMA0 Channel 0 Length / Transfer Control LSW Register
#define DMA0_CH0_TC_MSW         *(ioport volatile unsigned *)0x0c05    // DMA0 Channel 0 Transfer Control MSW Register
//**************************************************************************
//        DMA0 Ch1 Register Addresses
//**************************************************************************
#define DMA0_CH1_SRC_LSW        *(ioport volatile unsigned *)0x0c20    // DMA0 Channel 1 Source LSW Address
#define DMA0_CH1_SRC_MSW        *(ioport volatile unsigned *)0x0c21    // DMA0 Channel 1 Source MSW Address
#define DMA0_CH1_DST_LSW        *(ioport volatile unsigned *)0x0c22    // DMA0 Channel 1 Destination LSW Address
#define DMA0_CH1_DST_MSW        *(ioport volatile unsigned *)0x0c23    // DMA0 Channel 1 Destination MSW Address
#define DMA0_CH1_TC_LSW         *(ioport volatile unsigned *)0x0c24    // DMA0 Channel 1 Length / Transfer Control LSW Register
#define DMA0_CH1_TC_MSW         *(ioport volatile unsigned *)0x0c25    // DMA0 Channel 1 Transfer Control MSW Register
//**************************************************************************
//        DMA0 Ch2 Register Addresses
//**************************************************************************
#define DMA0_CH2_SRC_LSW        *(ioport volatile unsigned *)0x0c40    // DMA0 Channel 2 Source LSW Address
#define DMA0_CH2_SRC_MSW        *(ioport volatile unsigned *)0x0c41    // DMA0 Channel 2 Source MSW Address
#define DMA0_CH2_DST_LSW        *(ioport volatile unsigned *)0x0c42    // DMA0 Channel 2 Destination LSW Address
#define DMA0_CH2_DST_MSW        *(ioport volatile unsigned *)0x0c43    // DMA0 Channel 2 Destination MSW Address
#define DMA0_CH2_TC_LSW         *(ioport volatile unsigned *)0x0c44    // DMA0 Channel 2 Length / Transfer Control LSW Register
#define DMA0_CH2_TC_MSW         *(ioport volatile unsigned *)0x0c45    // DMA0 Channel 2 Transfer Control MSW Register

//**************************************************************************
//        DMA0 Ch3 Register Addresses
//**************************************************************************
#define DMA0_CH3_SRC_LSW        *(ioport volatile unsigned *)0x0c60    // DMA0 Channel 3 Source LSW Address
#define DMA0_CH3_SRC_MSW        *(ioport volatile unsigned *)0x0c61    // DMA0 Channel 3 Source MSW Address
#define DMA0_CH3_DST_LSW        *(ioport volatile unsigned *)0x0c62    // DMA0 Channel 3 Destination LSW Address
#define DMA0_CH3_DST_MSW        *(ioport volatile unsigned *)0x0c63    // DMA0 Channel 3 Destination MSW Address
#define DMA0_CH3_TC_LSW         *(ioport volatile unsigned *)0x0c64    // DMA0 Channel 3 Length / Transfer Control LSW Register
#define DMA0_CH3_TC_MSW         *(ioport volatile unsigned *)0x0c65    // DMA0 Channel 3 Transfer Control MSW Register

//****************************************************************************************
//****************************************************************************************
//   DMA Event Source Registers
//****************************************************************************************
#define DMA0_CH10_EVENT_SRC     *(ioport volatile unsigned *)0x1C1A
#define DMA0_CH32_EVENT_SRC     *(ioport volatile unsigned *)0x1C1B
#define DMA1_CH10_EVENT_SRC     *(ioport volatile unsigned *)0x1C1C     // DMA1 channel 0 and channel 1 Event Source Register
#define DMA1_CH32_EVENT_SRC     *(ioport volatile unsigned *)0x1C1D     // DMA1 channel 3 and channel 2 Event Source Register
#define DMA2_CH10_EVENT_SRC     *(ioport volatile unsigned *)0x1C36     // DMA2 channel 0 and channel 1 Event Source Register
#define DMA2_CH32_EVENT_SRC     *(ioport volatile unsigned *)0x1C37     // DMA2 channel 3 and channel 2 Event Source Register
#define DMA3_CH10_EVENT_SRC     *(ioport volatile unsigned *)0x1C38     // DMA3 channel 0 and channel 1 Event Source Register
#define DMA3_CH32_EVENT_SRC     *(ioport volatile unsigned *)0x1C39     // DMA3 channel 3 and channel 2 Event Source Register


//****************************************************************************************
//         DMA 1
//**************************************************************************
//        DMA1 Ch0 Register Addresses
//**************************************************************************
#define DMA1_CH0_SRC_LSW        *(ioport volatile unsigned *)0x0D00    // DMA1 Channel 0 Source LSW Address
#define DMA1_CH0_SRC_MSW        *(ioport volatile unsigned *)0x0D01    // DMA1 Channel 0 Source MSW Address
#define DMA1_CH0_DST_LSW        *(ioport volatile unsigned *)0x0D02    // DMA1 Channel 0 Destination LSW Address
#define DMA1_CH0_DST_MSW        *(ioport volatile unsigned *)0x0D03    // DMA1 Channel 0 Destination MSW Address
#define DMA1_CH0_TC_LSW         *(ioport volatile unsigned *)0x0D04    // DMA1 Channel 0 Length / Transfer Control LSW Register
#define DMA1_CH0_TC_MSW         *(ioport volatile unsigned *)0x0D05    // DMA1 Channel 0 Transfer Control MSW Register

//**************************************************************************
//        DMA1 Ch1 Register Addresses
//**************************************************************************
#define DMA1_CH1_SRC_LSW        *(ioport volatile unsigned *)0x0D20    // DMA1 Channel 1 Source LSW Address
#define DMA1_CH1_SRC_MSW        *(ioport volatile unsigned *)0x0D21    // DMA1 Channel 1 Source MSW Address
#define DMA1_CH1_DST_LSW        *(ioport volatile unsigned *)0x0D22    // DMA1 Channel 1 Destination LSW Address
#define DMA1_CH1_DST_MSW        *(ioport volatile unsigned *)0x0D23    // DMA1 Channel 1 Destination MSW Address
#define DMA1_CH1_TC_LSW         *(ioport volatile unsigned *)0x0D24    // DMA1 Channel 1 Length / Transfer Control LSW Register
#define DMA1_CH1_TC_MSW         *(ioport volatile unsigned *)0x0D25    // DMA1 Channel 1 Transfer Control MSW Register

//**************************************************************************
//        DMA1 Ch2 Register Addresses
//**************************************************************************
#define DMA1_CH2_SRC_LSW        *(ioport volatile unsigned *)0x0D40    // DMA1 Channel 2 Source LSW Address
#define DMA1_CH2_SRC_MSW        *(ioport volatile unsigned *)0x0D41    // DMA1 Channel 2 Source MSW Address
#define DMA1_CH2_DST_LSW        *(ioport volatile unsigned *)0x0D42    // DMA1 Channel 2 Destination LSW Address
#define DMA1_CH2_DST_MSW        *(ioport volatile unsigned *)0x0D43    // DMA1 Channel 2 Destination MSW Address
#define DMA1_CH2_TC_LSW         *(ioport volatile unsigned *)0x0D44    // DMA1 Destination Address
#define DMA1_CH2_TC_MSW         *(ioport volatile unsigned *)0x0D45    // DMA1 Channel 2 Transfer Control MSW Register

//**************************************************************************
//        DMA1 Ch3 Register Addresses
//**************************************************************************
#define DMA1_CH3_SRC_LSW        *(ioport volatile unsigned *)0x0D60    // DMA1 Channel 3 Source LSW Address
#define DMA1_CH3_SRC_MSW        *(ioport volatile unsigned *)0x0D61    // DMA1 Channel 3 Source MSW Address
#define DMA1_CH3_DST_LSW        *(ioport volatile unsigned *)0x0D62    // DMA1 Channel 3 Destination LSW Address
#define DMA1_CH3_DST_MSW        *(ioport volatile unsigned *)0x0D63    // DMA1 Channel 3 Destination MSW Address
#define DMA1_CH3_TC_LSW         *(ioport volatile unsigned *)0x0D64    // DMA1 Destination Address
#define DMA1_CH3_TC_MSW         *(ioport volatile unsigned *)0x0D65    // DMA1 Channel 0 Transfer Control MSW Register


//****************************************************************************************
//   DMA Interrupt
//****************************************************************************************
#define DMA_IFR     *(ioport volatile unsigned *)0x1C30    // DMA Interrupt Flag Register
#define DMA_MSK     *(ioport volatile unsigned *)0x1C31    // DMA Interrupt Mask Flag Register

// DMA_IFR             Address:  0x1C30 (IO space)       DMA Interrupt Flag Register
// +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
// |15:14:13:12|11:10: 9: 8| 7: 6: 5: 4| 3: 2: 1: 0|  <<== Bit Position
// +--+--+--+--+--+--+--+--+--+--+--+--+--+--:--:--+
// |rw:rw:rw:rw|rw:rw:rw:rw|rw:rw:rw:rw|rw:rw:rw:rw|  <<== Read / Write Access
// +--+--+--+--+--+--+--+--+--+--+--+--+--+--:--:--+
// | 0: 0: 0: 0| 0: 0: 0: 0| 0: 0: 0: 0| 0: 0: 0: 0|  <<== Reset Values
// +--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+--+
//   |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  |   (Writing a '1' clears the bit; writing a '0' has no effect)
//   |  |  |  |  |  |  |  |  |  |  |  |  |  |  |  +-- [0]  DMA0_CH0_INTF (0=no interrupt occured, 1=interrupt occured)
//   |  |  |  |  |  |  |  |  |  |  |  |  |  |  +----- [1]  DMA0_CH1_INTF (0=no interrupt occured, 1=interrupt occured)
//   |  |  |  |  |  |  |  |  |  |  |  |  |  +-------- [2]  DMA0_CH2_INTF (0=no interrupt occured, 1=interrupt occured)
//   |  |  |  |  |  |  |  |  |  |  |  |  +----------- [3]  DMA0_CH3_INTF (0=no interrupt occured, 1=interrupt occured)
//   |  |  |  |  |  |  |  |  |  |  |  |
//   |  |  |  |  |  |  |  |  |  |  |  +-------------- [4]  DMA1_CH0_INTF (0=no interrupt occured, 1=interrupt occured)
//   |  |  |  |  |  |  |  |  |  |  +----------------- [5]  DMA1_CH1_INTF (0=no interrupt occured, 1=interrupt occured)
//   |  |  |  |  |  |  |  |  |  +-------------------- [6]  DMA1_CH2_INTF (0=no interrupt occured, 1=interrupt occured)
//   |  |  |  |  |  |  |  |  +----------------------- [7]  DMA1_CH3_INTF (0=no interrupt occured, 1=interrupt occured)
//   |  |  |  |  |  |  |  |
//   |  |  |  |  |  |  |  +-------------------------- [8]  DMA2_CH0_INTF (0=no interrupt occured, 1=interrupt occured)
//   |  |  |  |  |  |  +----------------------------- [9]  DMA2_CH1_INTF (0=no interrupt occured, 1=interrupt occured)
//   |  |  |  |  |  +-------------------------------- [10] DMA2_CH2_INTF (0=no interrupt occured, 1=interrupt occured)
//   |  |  |  |  +----------------------------------- [11] DMA2_CH3_INTF (0=no interrupt occured, 1=interrupt occured)
//   |  |  |  |
//   |  |  |  +-------------------------------------- [12] DMA3_CH0_INTF (0=no interrupt occured, 1=interrupt occured)
//   |  |  +----------------------------------------- [13] DMA3_CH1_INTF (0=no interrupt occured, 1=interrupt occured)
//   |  +-------------------------------------------- [14] DMA3_CH2_INTF (0=no interrupt occured, 1=interrupt occured)
//   +----------------------------------------------- [15] DMA3_CH3_INTF (0=no interrupt occured, 1=interrupt occured)




//extern cregister volatile unsigned int IER0;

//****************************************************************************
//#define HPF_MODE 0//葤
#define HPF_MODE 1//葤

//20240925_TAC5112 Test	#define PLL_100M 1 //20200429
#define PLL_98M 1 //20240925_TAC5112 Test


#define XMIT_BUFF_SIZE 48//20200317

#define XMIT_L_BUFF_SIZE	48
#define XMIT_R_BUFF_SIZE	48

#define AUTO_RELOAD		0
#define SINE_WAVE		2

//20200317 Appended follows
#define longUPPERLIMIT 1073741823 //vZKvȂ̂ŁA32bit{MAX(2147483647)̔
#define longBOTTOMLIMIT -1073741824 //vZKvȂ̂ŁA32bit[MAX(-2147483648)̔

#define shortUPPERLIMIT 32767 //16bit{MAX(65535)̔
#define shortBOTTOMLIMIT -32768 //16bit[MAX(-65536)̔
#define CHANMAX 2//FilterCh
#define STAGEMAX 3//Filteȑd
#define LEFTCH 0//FilterChԍ
#define RIGHTCH 1//FilterChԍ
//20200317 Appended above

//GR[p萔

#define FXPARAMMAX 5
//20200410	#define DELAYLINE_MAX   8600 //Ƃ肠A384 x 100
#define DELAYLINE_MAX   6100 //
#define OD_DELAYLINE_MAX   4000 //20200507
#define HALFDELAYSPAN_b 3000
#define HALFDELAYSPAN_a 4000
#define CHRS_DELAYLINE_MAX 1024

#define XFADELEVMAX 32767
//20210406	#define LFOVALUEMAX 32767
#define LFOVALUEMAX 65535

/*20200319*/ //TAC5112_2024
// -3dB 1 KHz sine sampled at 48 KHz
Int16 XmitL_Sine_1K[XMIT_L_BUFF_SIZE] = {
0x0000,0x0B6C,0x16A6,0x217C,0x2BC0,0x3544,0x3DDF,0x456B, 
0x4BC7,0x50D7,0x5485,0x56C0,0x5780,0x56C0,0x5485,0x50D7, 
0x4BC7,0x456B,0x3DDF,0x3544,0x2BC0,0x217C,0x16A6,0x0B6C, 
0x0000,0xF494,0xE95A,0xDE84,0xD440,0xCABC,0xC221,0xBA95, 
0xB439,0xAF29,0xAB7B,0xA940,0xA880,0xA940,0xAB7B,0xAF29, 
0xB439,0xBA95,0xC221,0xCABC,0xD440,0xDE84,0xE95A,0xF494
};

// -3dB 1 KHz sine sampled at 48 KHz
Int16 XmitR_Sine_1K[XMIT_R_BUFF_SIZE] = {
0x0000,0x0B6C,0x16A6,0x217C,0x2BC0,0x3544,0x3DDF,0x456B, 
0x4BC7,0x50D7,0x5485,0x56C0,0x5780,0x56C0,0x5485,0x50D7, 
0x4BC7,0x456B,0x3DDF,0x3544,0x2BC0,0x217C,0x16A6,0x0B6C, 
0x0000,0xF494,0xE95A,0xDE84,0xD440,0xCABC,0xC221,0xBA95, 
0xB439,0xAF29,0xAB7B,0xA940,0xA880,0xA940,0xAB7B,0xAF29, 
0xB439,0xBA95,0xC221,0xCABC,0xD440,0xDE84,0xE95A,0xF494
};

//-3 dB 2 KHz sine sampled at 48 KHz
Int16 XmitR_Sine_2K[XMIT_R_BUFF_SIZE] = {
0x0000,0x16A6,0x2BC0,0x3DDF,0x4BC7,0x5485,0x5780,0x5485, 
0x4BC7,0x3DDF,0x2BC0,0x16A6,0x0000,0xE95A,0xD440,0xC221, 
0xB439,0xAB7B,0xA880,0xAB7B,0xB439,0xC221,0xD440,0xE95A,
0x0000,0x16A6,0x2BC0,0x3DDF,0x4BC7,0x5485,0x5780,0x5485, 
0x4BC7,0x3DDF,0x2BC0,0x16A6,0x0000,0xE95A,0xD440,0xC221, 
0xB439,0xAB7B,0xA880,0xAB7B,0xB439,0xC221,0xD440,0xE95A
};
/*20200319*/ //TAC5112_2024

//20190412
Int16 RcvL1[XMIT_BUFF_SIZE];
Int16 RcvR1[XMIT_BUFF_SIZE];
Int16 RcvL2[XMIT_BUFF_SIZE];
Int16 RcvR2[XMIT_BUFF_SIZE];

//Int16 XmitL1[XMIT_BUFF_SIZE];
//Int16 XmitR1[XMIT_BUFF_SIZE];
//Int16 XmitL2[XMIT_BUFF_SIZE];
//Int16 XmitR2[XMIT_BUFF_SIZE];

Int16 TrsL1[XMIT_BUFF_SIZE];
Int16 TrsR1[XMIT_BUFF_SIZE];
Int16 TrsL2[XMIT_BUFF_SIZE];
Int16 TrsR2[XMIT_BUFF_SIZE];

Int16 AnotherChBuff_1[XMIT_BUFF_SIZE];//20200323
Int16 AnotherChBuff_2[XMIT_BUFF_SIZE];//20200323

Int16 TrsLchBuff_1[XMIT_BUFF_SIZE];
Int16 TrsLchBuff_2[XMIT_BUFF_SIZE];
Int16 TrsRchBuff_1[XMIT_BUFF_SIZE];
Int16 TrsRchBuff_2[XMIT_BUFF_SIZE];



Uint32 RxL1_DMA_address;
Uint32 RxL2_DMA_address;
Uint32 RxR1_DMA_address;
Uint32 RxR2_DMA_address;
Uint32 TxL1_DMA_address;
Uint32 TxL2_DMA_address;
Uint32 TxR1_DMA_address;
Uint32 TxR2_DMA_address;

volatile Uint16 CurrentRxL_DMAChannel;
volatile Uint16 CurrentRxR_DMAChannel;
volatile Uint16 CurrentTxL_DMAChannel;
volatile Uint16 CurrentTxR_DMAChannel;

volatile Uint16 InputOperatingForL;
volatile Uint16 InputOperatingForR;
volatile Uint16 OutputOperatingForL;
volatile Uint16 OutputOperatingForR;

//20200417	volatile unsigned char TrsBuff_1Prepared = 0;//20200416_TrsDataPrepared
//20200417	volatile unsigned char TrsBuff_2Prepared = 0;//20200416_TrsDataPrepared

/*20200416*/
//Filter1ϐ
float Q, f, Scale;// 1 >= Q >= 0;

//short sCutoff = f*65535, sfs = fs*65535;
short sQ;
short sf;
short sScale;

typedef struct{
	volatile short High;
	volatile short Band;
	volatile short Low;
	volatile short Notch;
} FILTER_OUT;
volatile FILTER_OUT Filter_Out[STAGEMAX][CHANMAX];

short OnePoleLPF_Output[STAGEMAX][CHANMAX];
long OnePoleLPF_OutputxX[STAGEMAX][CHANMAX];
float fA;
short sa;
int TCount[STAGEMAX][CHANMAX];

volatile short ScratchPad16;
volatile unsigned short ScratchPadTest;
/*20200416*/

interrupt void DMA_Isr(void);

void buff_copy(Int16 *input, Int16 *output, Int16 size);
void HPFilter(Int16 *input, Int16 *output, Int16 size, int ch);
void LPFilter(Int16 *input, Int16 *output, Int16 *suboutput, Int16 size, int ch);
void StateVariableFiter(short Input, int ch, int stage);
void OnePoleLPF(short Input, int ch, int stage);


//GR[pϐ
short gDelayLine_Lch[DELAYLINE_MAX];
short gDelayLine_Rch[DELAYLINE_MAX];

//IN^[uVtgp
short gOD_DelayLine[OD_DELAYLINE_MAX];

//R[Xp
short gChrs_DelayLine[CHRS_DELAYLINE_MAX];

//20200319	unsigned short gCurrentADRS;
volatile short gCurrentADRS = 0;
volatile short gOD_CurrentADRS = 0;
volatile short gChrs_CurrentADRS = 0;

//20200415	volatile char gBothChSentout;

char DrySft[FXPARAMMAX];
short gDirectSrc;//ϲ޲ڸĐM
short gFxSend;//̪ľސM
short gFB_L, gFB_R, XgFB_L, XgFB_R;//ިڲ̨ޯA۽ĘޯM
//20170316long gDelayIn_L, gDelayIn_R, gDelayOut_L, gDelayOut_R;//L/Rިڲn̓o
//20181211 long gDelayIn_L, gDelayIn_R, gDelayOut_L, gDelayOut_R, gDelayOut_L2, gDelayOut_R2;//L/Rިڲn̓o
short gDelayIn_L, gDelayIn_R, gDelayOut_L1, gDelayOut_R1, gDelayOut_L2, gDelayOut_R2, gDelayOut_L3, gDelayOut_R3;//L/Rިڲn̓o
short gOD_DelayIn, gOctDwnOut_b, gOctDwnOut_a;
short gChrs_DelayIn, gChrs_DelayOut_1, gChrs_DelayOut_2;
short gChrs_DelayLength_1, gChrs_DelayLength_2;
short gMix_L, gMix_R;//̪Ċ֐ƂL/Rւ̍ŏIo
unsigned char gFxParamNum;

//long gMainVal, gMicTotalVal;//ŏIoفAϲİ
//long gFxVolVal;//̪Đ̏o
//long gMicVolVal;//ϲحщZ
short gFxSendVal;//̏ϲ1,2̴̪
short gFBVal_L, gFBVal_R;//L/Rn eX̨ޯ
short gXFBVal_L, gXFBVal_R;//L/Rn eX݂ւ̨ޯ
//long gFBVol;//L/Rǫޯ
//unsigned short gDelayLength_L, gDelayLength_R;//L/Rn eXިڲײ݂̒
short gDelayTime;//L/Roިڲײ݂̒

typedef struct{
//20200319	int gDelayLength_L;
//20200319	int gDelayLength_R;
	short gDelayLength_L;
	short gDelayLength_R;
//	char DrySft;
	char FxSendSft;
    char FB_LSft;
    char FB_RSft;
    char XFB_LSft;
    char XFB_RSft;
} FX_PARAM;

FX_PARAM gFxParam1[FXPARAMMAX];
FX_PARAM gFxParam2[FXPARAMMAX];//20170316
FX_PARAM gFxParam3[FXPARAMMAX];//20181210

short HalfDelaySpan_b = HALFDELAYSPAN_b;//20200408
volatile short HalfDelayPoint_Tail_b = 0;//20200408
volatile short HalfDelayPoint_Frnt_b = 0;//20200408
volatile unsigned short HalfDelayCount_b = 0;//20200408

volatile unsigned short XFadeLevel_b = 0;//20200408
//20200415	volatile unsigned short XFadeStep_L;//20200408


short HalfDelaySpan_a = HALFDELAYSPAN_a;//20200405
volatile short HalfDelayPoint_Tail_a = 0;//20200405
volatile short HalfDelayPoint_Frnt_a = 0;//20200405
volatile unsigned short HalfDelayCount_a = 0;//20200405

volatile unsigned short XFadeLevel_a = 0;//20200405
//20200415	volatile unsigned short XFadeStep_R;//20200405

//20200415	volatile char OperationTerm = 0;//20200406

volatile short LFOValue;//20210325
volatile char LFODir;//20210325

//GR[֐
void ReverbInit(void);
void PutDataToDelayLine(short LchData, short RchData);
void PutDataToOD_DelayLine(short Data);
void PutDataToChrs_DelayLine(short Data);//20210402

short LFO(short Turm);

//20200319	void SetDataToDelayOut(unsigned short RecentADRS);//���ݓ_��RecentADRS�ɑ΂���DelayLength�����ߋ����ް��l��Ԃ��B
//20200403void SetDataToDelayOut(short RecentADRS);//���ݓ_��RecentADRS�ɑ΂���DelayLength�����ߋ����ް��l��Ԃ��B
void SetDataToDelayOut(void);//ݓ_gCurrentADRSɑ΂DelayLengthߋްlԂB
void EchoFx(short gDirectSrc);//�̪�āi32bit�����j�����������ʂ��ް��i32bit�Łj��gMix_L/R,��gDelayOut_L/R�ɓ��͂���B


#define TAC5112_I2C_ADDR	0x50 //20240918
Uint8 code_TAC5112; //20240918
//20240924 Uint8* cmd; //20240918

//****************************************************************************
void PLL_98MHz(void)
{
// PLL set up from RTC
    // bypass PLL
    CONFIG_MSW = 0x0;

#if (PLL_100M ==1)
    PLL_CNTL2 = 0x8000;
    PLL_CNTL4 = 0x0000;
    PLL_CNTL3 = 0x0806;
//20210412test    PLL_CNTL1 = 0x8BE8; //PG1.4: 0x82FA;
	PLL_CNTL1 = 0x82E8; //PG1.4: 0x82FA;
//20210412test	PLL_CNTL1 = 0x8174; //PG1.4: 0x82FA;
    
#elif (PLL_12M ==1)
    PLL_CNTL2 = 0x8000;
    PLL_CNTL4 = 0x0200;
    PLL_CNTL3 = 0x0806;
    PLL_CNTL1 = 0x82ED;
#elif (PLL_98M ==1)    
	PLL_CNTL2 = 0x8000;
    PLL_CNTL4 = 0x0000;
    PLL_CNTL3 = 0x0806;

//20240925_TAC5112 Test	    PLL_CNTL1 = 0x82ED;
PLL_CNTL1 = 0x807C;//20240925
    
#endif
    while ( (PLL_CNTL3 & 0x0008) == 0);
    // Switch to PLL clk
    CONFIG_MSW = 0x1;
}

//****************************************************************************
void InitSystem(void)
{
	Uint16 i;
// PLL set up from RTC
    // bypass PLL
    CONFIG_MSW = 0x0;

#if (PLL_100M ==1)
    PLL_CNTL2 = 0x8000;
    PLL_CNTL4 = 0x0000;
    PLL_CNTL3 = 0x0806;
    PLL_CNTL1 = 0x8BE8; //PG1.4: 0x82FA;
    
#elif (PLL_12M ==1)
    PLL_CNTL2 = 0x8000;
    PLL_CNTL4 = 0x0200;
    PLL_CNTL3 = 0x0806;
    PLL_CNTL1 = 0x82ED;
#elif (PLL_98M ==1)    
    PLL_CNTL2 = 0x8000;
    PLL_CNTL4 = 0x0000;
    PLL_CNTL3 = 0x0806;
    PLL_CNTL1 = 0x82ED;

#elif (PLL_40M ==1)        
    PLL_CNTL2 = 0x8000;
    PLL_CNTL4 = 0x0300;
    PLL_CNTL3 = 0x0806;
    PLL_CNTL1 = 0x8262;    
#endif

    while ( (PLL_CNTL3 & 0x0008) == 0);
    // Switch to PLL clk
    CONFIG_MSW = 0x1;

// clock gating
// enable all clocks
    IDLE_PCGCR = 0x0;
    IDLE_PCGCR_MSW = 0xFF84;
    

// reset peripherals
    PER_RSTCOUNT = 0x02;
    PER_RESET = 0x00fb;    
    for (i=0; i< 0xFFF; i++);
}

//****************************************************************************
void ConfigPort(void)
{
    Int16 i;
    //  configure ports
    PERIPHSEL0 = 0x6900;        // parallel port: mode 6, serial port1: mode 2 

//20200417    for (i=0; i< 0xFFF; i++);
    for (i=0; i< 0xFFFF; i++);
}

//****************************************************************************
void SYS_GlobalIntEnable(void)
{
    asm(" BIT (ST1, #ST1_INTM) = #0");
}
//****************************************************************************
void SYS_GlobalIntDisable(void)
{
    asm(" BIT (ST1, #ST1_INTM) = #1");
}

//****************************************************************************
//Int16 AIC3204_rget(  Uint16 regnum, Uint16* regval )
Int16 AIC3204_rget(  Uint16 regnum, Uint8* regval )
{
    Int16  retcode = 0;
//20240925_TAC5112 Test    Uint16 cmd[2];
Uint8 cmd[2];//20240925_TAC5112 Test

//20240925_TAC5112 Test    cmd[0] = regnum & 0x007F;       // 7-bit Device Register
    cmd[0] = (Uint8)(regnum & 0x007F);       // 7-bit Device Register
    cmd[1] = 0;

    retcode |= USBSTK5505_I2C_write( (Uint16)AIC3204_I2C_ADDR, (Uint8*)cmd, 1 );
    retcode |= USBSTK5505_I2C_read( (Uint16)AIC3204_I2C_ADDR, (Uint8*)cmd, 1 );

    *regval = cmd[0];
    USBSTK5505_wait( 10 );
    return retcode;
}
//****************************************************************************
//Int16 TAC5112_rget(  Uint16 regnum, Uint16* regval )
//Int16 TAC5112_rget(  Uint16 regnum, Uint8* regval )
Int8 TAC5112_rget(  Uint16 regnum )
{
    Int16  retcode = 0;
//20240925_TAC5112 Test    Uint16 cmd[2];
Uint8 cmd[2];//20240925_TAC5112 Test

//20240925_TAC5112 Test    cmd[0] = regnum & 0x007F;       // 7-bit Device Register
    cmd[0] = (Uint8)(regnum & 0x007F);       // 7-bit Device Register
    cmd[1] = 0;

    retcode |= USBSTK5505_I2C_write( (Uint16)TAC5112_I2C_ADDR, (Uint8*)cmd, 1 );
    retcode |= USBSTK5505_I2C_read( (Uint16)TAC5112_I2C_ADDR, (Uint8*)cmd, 1 );

//20240925_TAC5112 Test    *regval = cmd[0];
    USBSTK5505_wait( 10 );
    //20240925_TAC5112 Test return retcode;
    return cmd[0];//20240925_TAC5112 Test
}
//****************************************************************************
Int16 AIC3204_rset( Uint16 regnum, Uint16 regval )
{
	unsigned char cmd[2];
	cmd[0] = regnum & 0x007F;	// 7-bit Device Address
	cmd[1] = regval;			// 8-bit Register Data
	
    USBSTK5505_waitusec( 300 );
	
	return USBSTK5505_I2C_write( AIC3204_I2C_ADDR, cmd, 2 );//eZDsp5535̏ꍇABSL EZDSP5535_I2C_write( AIC3204_I2C_ADDR, cmd, 2 )B
}
//****************************************************************************
Int16 TAC5112_rset( Uint16 regnum, Uint16 regval )
{
	unsigned char cmd[2];
	cmd[0] = regnum & 0x007F;	// 7-bit Device Address
	cmd[1] = regval;			// 8-bit Register Data
	
    USBSTK5505_waitusec( 300 );
	
	return USBSTK5505_I2C_write( TAC5112_I2C_ADDR, cmd, 2 );//eZDsp5535̏ꍇABSL EZDSP5535_I2C_write( AIC3204_I2C_ADDR, cmd, 2 )B
}
//****************************************************************************
void AIC3204_Init(void)
{
	SYS_EXBUSSEL |= 0x0020;		// Select A20/GPIO26 as GPIO26
	USBSTK5505_GPIO_init();
	USBSTK5505_GPIO_setDirection(GPIO26, GPIO_OUT);
	USBSTK5505_GPIO_setOutput( GPIO26, 1 );		// Take AIC3201 chip out of reset
	USBSTK5505_I2C_init( );						// Initialize I2C, eZDsp5535̏ꍇBSL̒EZDSP5535_I2C_init( )B

	/* Configure Serial Bus */
//TAC5112_20240803	SYS_EXBUSSEL |= 0x0100;		// Configure Serial bus 0 for I2S0
SYS_EXBUSSEL |= 0x0400;	//TAC5112_20240803	// Configure Serial bus 0 for I2S2

	/* Configure AIC3204 */

/*TAC5112_20241004
	AIC3204_rset( 0, 0 );		// Select page 1
	AIC3204_rset( 1, 1 );		// Reset codec
	AIC3204_rset( 0, 1 );		// Point to page 1
	AIC3204_rset( 1, 8 );		// Disable crude AVDD generation from DVDD
	AIC3204_rset( 2, 1 );		// Enable Analog Blocks, use LDO power
	AIC3204_rset( 0, 0 );
	// PLL and Clocks config and Power Up
	AIC3204_rset( 27, 0x0d );	// BCLK and WCLK is set as o/p to AIC3204(Master)
	AIC3204_rset( 28, 0x00 );	// Data ofset = 0
	AIC3204_rset( 4, 3 );		// PLL setting: PLLCLK <- MCLK, CODEC_CLKIN <-PLL CLK
	AIC3204_rset( 6, 8 );		// PLL setting: J=8
	AIC3204_rset( 7, 15 );		// PLL setting: HI_BYTE(D)
	AIC3204_rset( 8, 0xdc ); 	// PLL setting: LO_BYTE(D)
	AIC3204_rset( 30, 0x88 );	// For 32 bit clocks per frame in Master mode ONLY
								// BCLK=DAC_CLK/N =(12288000/8) = 1.536MHz = 32*fs
//20210412test	AIC3204_rset( 30, 0x90 );	// For 32 bit clocks per frame in Master mode ONLY
//20210412test								// BCLK=DAC_CLK/N =(12288000/16(0x10=.0x90)) = 768kHz = 32*fs
//20210412test	AIC3204_rset( 30, 0x98 );	// For 32 bit clocks per frame in Master mode ONLY
//20210412test								// BCLK=DAC_CLK/N =(12288000/24(0x18=.0x98)) = 576kHz = 32*fs
	AIC3204_rset( 5, 0x91 );	//?;PLL setting: Power up PLL, P=1 and R=1
	AIC3204_rset( 13, 0 );		// Hi_Byte(DOSR) for DOSR = 128 decimal or 0x0080 DAC oversamppling
	AIC3204_rset( 14, 0x80 );	// Lo_Byte(DOSR) for DOSR = 128 decimal or 0x0080
	AIC3204_rset( 20, 0x80 );	// AOSR for AOSR = 128 decimal or 0x0080 for decimation filters 1 to 6
	AIC3204_rset( 11, 0x88 );	// Power up NDAC and set NDAC value to 8
	AIC3204_rset( 12, 0x82 );	// Power up MDAC and set MDAC value to 2
	AIC3204_rset( 18, 0x88 );	// Power up NADC and set NADC value to 8
	AIC3204_rset( 19, 0x82 );	// Power up MADC and set MADC value to 2
	// DAC ROUTING and Power Up
	AIC3204_rset( 0, 1 );		// Select page 1
	AIC3204_rset( 0x0c, 8 );	// LDAC AFIR routed to HPL
	AIC3204_rset( 0x0d, 8 );	// RDAC AFIR routed to HPR
	AIC3204_rset( 0, 0 );		// Select page 0
	AIC3204_rset( 64, 2 );		// Left vol=right vol
	AIC3204_rset( 65, 0);		// Left DAC gain to 0dB VOL; Right tracks Left
	AIC3204_rset( 63, 0xd4 );	// Power up left,right data paths and set channel
	AIC3204_rset( 0, 1 );		// Select page 1
	AIC3204_rset( 0x10, 0x3a );	// Unmute HPL , -6dB gain
	AIC3204_rset( 0x11, 0x3a );	// Unmute HPR , -6dB gain
	AIC3204_rset( 9, 0x30 );	// Power up HPL,HPR
	AIC3204_rset( 0, 0 );		// Select page 0
	USBSTK5505_wait( 100 );	// waitAeZDsp5535̏ꍇ́ABSL EZDSP5535_waitusec( 300 )ȂǂB
	// ADC ROUTING and Power Up
	AIC3204_rset( 0, 1 );		// Select page 1
	AIC3204_rset( 0x34, 0x30 );	// STEREO 1 Jack
								// IN2_L to LADC_P through 40 kohm
	AIC3204_rset( 0x37, 0x30 );	// IN2_R to RADC_P through 40 kohmm
	AIC3204_rset( 0x36, 3 );	// CM_1 (common mode) to LADC_M through 40 kohm
	AIC3204_rset( 0x39, 0xc0 );	// CM_1 (common mode) to RADC_M through 40 kohm

//20200509	AIC3204_rset( 0x3b, 0 );	// MIC_PGA_L unmute(for Line_Level)
//20200509	AIC3204_rset( 0x3c, 0 );	// MIC_PGA_R unmute(for Line_Level)
AIC3204_rset( 0x3b, 60 );	// MIC_PGA_L +30dB(for Mic_Level)
AIC3204_rset( 0x3c, 60 );	// MIC_PGA_R +30dB(for Mic_Level)
//20200410AIC3204_rset( 0x33, 0x40 );	// MICBIAS Powered Up BIAS = LDOIN

	AIC3204_rset( 0, 0 );		// Select page 0
	AIC3204_rset( 0x51, 0xc0 );	// Powerup Left and Right ADC
	AIC3204_rset( 0x52, 0 );	// Unmute Left and Right ADC
    
	AIC3204_rset( 0, 0 );
	USBSTK5505_wait( 200 );	// WaitAeZDsp5535̏ꍇ́ABSL EZDSP5535_waitusec( 300 )ȂǂB
TAC5112_20241004*/

}
//****************************************************************************
void TAC5112_Init(void)
{
	SYS_EXBUSSEL |= 0x0020;		// Select A20/GPIO26 as GPIO26
	USBSTK5505_GPIO_init();
	USBSTK5505_GPIO_setDirection(GPIO26, GPIO_OUT);
	USBSTK5505_GPIO_setOutput( GPIO26, 1 );		// Take AIC3201 chip out of reset
	USBSTK5505_I2C_init( );						// Initialize I2C, eZDsp5535̏ꍇBSL̒EZDSP5535_I2C_init( )B

	/* Configure Serial Bus */
//TAC5112_20240803	SYS_EXBUSSEL |= 0x0100;		// Configure Serial bus 0 for I2S0
SYS_EXBUSSEL |= 0x0400;	//TAC5112_20240926	// Configure Serial bus 0 for I2S2

	// Configure TAC5112 as follows


	USBSTK5505_wait( 200 );	// WaitAeZDsp5535̏ꍇ́ABSL EZDSP5535_waitusec( 300 )ȂǂB

}
//################################################################################
void setDMA_address(void)
{
// double buffer with filtering    
	RxL1_DMA_address = (Uint32)RcvL1;   
	RxL1_DMA_address = (RxL1_DMA_address<<1) + 0x10000;		// change word address to byte address and add DARAM offset for DMA

	RxL2_DMA_address = (Uint32)RcvL2;   
	RxL2_DMA_address = (RxL2_DMA_address<<1) + 0x10000;		// change word address to byte address and add DARAM offset for DMA

	RxR1_DMA_address = (Uint32)RcvR1;   
	RxR1_DMA_address = (RxR1_DMA_address<<1) + 0x10000;		// change word address to byte address and add DARAM offset for DMA

	RxR2_DMA_address = (Uint32)RcvR2;   
	RxR2_DMA_address = (RxR2_DMA_address<<1) + 0x10000;		// change word address to byte address and add DARAM offset for DMA

	TxL1_DMA_address = (Uint32)TrsL1;   
	TxL1_DMA_address = (TxL1_DMA_address<<1) + 0x10000;		// change word address to byte address and add DARAM offset for DMA

	TxL2_DMA_address = (Uint32)TrsL2;   
	TxL2_DMA_address = (TxL2_DMA_address<<1) + 0x10000;		// change word address to byte address and add DARAM offset for DMA
    
	TxR1_DMA_address = (Uint32)TrsR1;   
	TxR1_DMA_address = (TxR1_DMA_address<<1) + 0x10000;		// change word address to byte address and add DARAM offset for DMA
    
	TxR2_DMA_address = (Uint32)TrsR2;   
	TxR2_DMA_address = (TxR2_DMA_address<<1) + 0x10000;		// change word address to byte address and add DARAM offset for DMA      
}
//****************************************************************************
void I2S_Start(void)
{
	/* I2S settings */
//TAC5112_20240803	I2S0_SRGR = 0x0;	//1/8fs, 1/2Clk_Divide, by SampleRate Generate Reg.
I2S2_SRGR = 0x0015;	//FS=1.536MHz x 1/32(=48kHz), BCLK=98MHz x 1/64(=1.536MHz)Clk_Divide, by SampleRate Generate Reg.

//20200331test	I2S0_SRGR = 0x4;	//1/8fs, 1/8Clk_Divide, by SampleRate Generate Reg.
	//I2S0_CR = 0x8010;	// 16-bit word "10"(bit5`24h), slave, enable I2C

//TAC5112_20240803	I2S0_CR = 0x8090;	// 16-bit word "10"(bit5`24h), slave, Pack, enable I2C
I2S2_CR = 0x8092;	// 16-bit word "10"(bit5`24h), master, Pack, enable I2C

}
//****************************************************************************
void set_dma0_ch0_stop(void)
{
	Uint16 temp;
//  DMA stop
//TAC5112_20240926	temp = DMA0_CH0_TC_MSW;
temp = DMA1_CH0_TC_MSW;//TAC5112_20240803
	temp &= 0x7FFF;
//TAC5112_20240926	DMA0_CH0_TC_MSW = temp;    
DMA1_CH0_TC_MSW = temp;//TAC5112_20240926    
}
//****************************************************************************
void set_dma0_ch1_stop(void)
{
	Uint16 temp;
//  DMA stop
//TAC5112_20240926	temp = DMA0_CH1_TC_MSW;
temp = DMA1_CH1_TC_MSW;//TAC5112_20240926
	temp &= 0x7FFF;
//TAC5112_20240926	DMA0_CH1_TC_MSW = temp;    
DMA1_CH1_TC_MSW = temp;//TAC5112_20240926    
}
//################################################################################
void set_dma0_ch2_stop(void)
{
Uint16 temp;
//  DMA stop
//TAC5112_20240926	temp = DMA0_CH2_TC_MSW;
temp = DMA1_CH2_TC_MSW;//TAC5112_20240926
	temp &= 0x7FFF;
//TAC5112_20240926	DMA0_CH2_TC_MSW = temp;    
DMA1_CH2_TC_MSW = temp;//TAC5112_20240926    
}
//################################################################################
void set_dma0_ch3_stop(void)
{
Uint16 temp;
//  DMA stop
//TAC5112_20240926	temp = DMA0_CH3_TC_MSW;
temp = DMA1_CH3_TC_MSW;//TAC5112_20240926
	temp &= 0x7FFF;
//TAC5112_20240926	DMA0_CH3_TC_MSW = temp;    
DMA1_CH3_TC_MSW = temp;//TAC5112_20240926    
}
//****************************************************************************
Uint16 set_dma0_ch0_i2s0_Lout(void)
{
	Uint16 temp; 
	Uint32 add;//TAC5112_20240926 for Sine_1kHz Out

//TAC5112_20240926	DMA0_CH0_TC_LSW = XMIT_BUFF_SIZE * 2;
DMA1_CH0_TC_LSW = XMIT_BUFF_SIZE * 2;

//TAC5112_20240926	DMA0_CH0_TC_MSW = 0x2204;	//src incre, destination fix, sync, No auto  
DMA1_CH0_TC_MSW = 0x2204;	//TAC5112_20240926//src incre, destination fix, sync, No auto  
//  DMA0_CH0_TC_MSW = 0x3204;	//src incre, destination fix, sync, auto  
	            
//TAC5112_20240926	temp = DMA0_CH10_EVENT_SRC;
temp = DMA1_CH10_EVENT_SRC;//TAC5112_20240926
//TAC5112_20240926	DMA0_CH10_EVENT_SRC = temp | 0x0001;	// I2S0 transmit event
DMA1_CH10_EVENT_SRC = temp | 0x0001;//TAC5112_20240926	// I2S0 transmit event
    
/*20200317 //TAC5112_20240803
	add = (Uint32)XmitL_Sine_1K;        
	add = (add<<1) + 0x10000;	// change word address to byte address and add DARAM offset for DMA
   
//TAC5112_20240926	DMA0_CH0_SRC_LSW = (Uint16)add;
DMA1_CH0_SRC_LSW = (Uint16)add;//TAC5112_20240926
//TAC5112_20240926	DMA0_CH0_SRC_MSW = 0xFFFF & (add >> 16);
DMA1_CH0_SRC_MSW = 0xFFFF & (add >> 16);//TAC5112_20240926
20200317*/ //TAC5112_20240803
//20200317
	DMA0_CH0_SRC_LSW = (Uint16)TxL1_DMA_address;
	DMA0_CH0_SRC_MSW = 0xFFFF & (TxL1_DMA_address >> 16);


    
//TAC5112_20240803	DMA0_CH0_DST_LSW = 0x2808;	// is20 transmit left data register lsw
//TAC5112_20240926	DMA0_CH0_DST_LSW = 0x2A08;	// i2s2 transmit left data register lsw
DMA1_CH0_DST_LSW = 0x2A08;//TAC5112_20240926	// i2s2 transmit left data register lsw
    
//  DMA starts   
//TAC5112_20240926	temp = DMA0_CH0_TC_MSW;
temp = DMA1_CH0_TC_MSW;//TAC5112_20240926
	temp |= 0x8000;
//TAC5112_20240926	DMA0_CH0_TC_MSW = temp;
DMA1_CH0_TC_MSW = temp;//TAC5112_20240926

	return 0;
}
//****************************************************************************
Uint16 set_dma0_ch1_i2s0_Rout(void)
{	
	Uint16 temp;
	Uint32 add;//TAC5112_20240803

//TAC5112_20240926	DMA0_CH1_TC_LSW = XMIT_BUFF_SIZE * 2;
DMA1_CH1_TC_LSW = XMIT_BUFF_SIZE * 2;//TAC5112_20240926

//TAC5112_20240926	DMA0_CH1_TC_MSW = 0x2204;	//src incre, destination fix, sync, No auto  
DMA1_CH1_TC_MSW = 0x2204;//TAC5112_20240926	//src incre, destination fix, sync, No auto  
//  DMA0_CH1_TC_MSW = 0x3204;	//src incre, destination fix, sync, auto  
	
//TAC5112_20240926	temp = DMA0_CH10_EVENT_SRC;
temp = DMA1_CH10_EVENT_SRC;//TAC5112_20240926
//TAC5112_20240926	DMA0_CH10_EVENT_SRC = temp | 0x0100;	// I2S0 transmit event
DMA1_CH10_EVENT_SRC = temp | 0x0100;//TAC5112_20240926	// I2S0 transmit event
	
/*20200317 //TAC5112_20240803
	//add = (Uint32)XmitR_Sine_1K;        
	add = (Uint32)XmitR_Sine_2K;        
	add = (add<<1) + 0x10000;	// change word address to byte address and add DARAM offset for DMA
    
//TAC5112_20240926	DMA0_CH1_SRC_LSW = (Uint16)add;
DMA1_CH1_SRC_LSW = (Uint16)add;//TAC5112_20240926
//TAC5112_20240926	DMA0_CH1_SRC_MSW = 0xFFFF & (add >> 16);
DMA1_CH1_SRC_MSW = 0xFFFF & (add >> 16);//TAC5112_20240926
20200317*/ //TAC5112_20240803
//20200317
	DMA0_CH1_SRC_LSW = (Uint16)TxR1_DMA_address;
	DMA0_CH1_SRC_MSW = 0xFFFF & (TxR1_DMA_address >> 16);


//TAC5112_20240803	DMA0_CH1_DST_LSW = 0x280C;	// is20 transmit right data register lsw
//TAC5112_20240926	DMA0_CH1_DST_LSW = 0x2A0C;	// i2s2 transmit right data register lsw
DMA1_CH1_DST_LSW = 0x2A0C;//TAC5112_20240926	// i2s2 transmit right data register lsw

// DMA starts    
//TAC5112_20240926	temp = DMA0_CH1_TC_MSW;
temp = DMA1_CH1_TC_MSW;//TAC5112_20240926
	temp |= 0x8000;
//TAC5112_20240926	DMA0_CH1_TC_MSW = temp;
DMA1_CH1_TC_MSW = temp;//TAC5112_20240926

	return 0;
}
//################################################################################
//Uint16 set_dma1_ch2_i2s2_Lin(void)
Uint16 set_dma0_ch2_i2s0_Lin(void)
{
Uint16 temp; 

//TAC5112_20240926	DMA0_CH2_TC_LSW = XMIT_BUFF_SIZE*2;
DMA1_CH2_TC_LSW = XMIT_BUFF_SIZE*2;//TAC5112_20240926

//	DMA0_CH2_TC_MSW = 0x3084;     //src fix, destination increase, sync, auto, int 
//TAC5112_20240926	DMA0_CH2_TC_MSW = 0x2084;     //src fix, destination increase,, No auto, int
DMA1_CH2_TC_MSW = 0x2084;//TAC5112_20240926     //src fix, destination increase,, No auto, int

//TAC5112_20240926	temp = DMA0_CH32_EVENT_SRC;
temp = DMA1_CH32_EVENT_SRC;//TAC5112_20240926
//TAC5112_20240926	DMA0_CH32_EVENT_SRC = temp | 0x0002;   // I2S0 receive event
DMA1_CH32_EVENT_SRC = temp | 0x0002;//TAC5112_20240926   // I2S0 receive event

//TAC5112_20240926	DMA0_CH2_DST_LSW = (Uint16)RxL1_DMA_address;
DMA1_CH2_DST_LSW = (Uint16)RxL1_DMA_address;//TAC5112_20240926
//TAC5112_20240926	DMA0_CH2_DST_MSW = 0xFFFF & (RxL1_DMA_address >> 16);
DMA1_CH2_DST_MSW = 0xFFFF & (RxL1_DMA_address >> 16);//TAC5112_20240926
    
//TAC5112_20240803    DMA0_CH2_SRC_LSW = 0x2828;      // is22 receive left data register lsw
//TAC5112_20240926	DMA0_CH2_SRC_LSW = 0x2A28;      // i2s2 receive left data register lsw
DMA1_CH2_SRC_LSW = 0x2A28;//TAC5112_20240926      // i2s2 receive left data register lsw

//20140617 T.Okutsu	DMA0_CH2_SRC_LSW = 0x2A28;      // is22 receive left data register lsw
//TAC5112_20240926	DMA0_CH2_SRC_MSW = 0;
DMA1_CH2_SRC_MSW = 0;//TAC5112_20240926

	//CurrentRxL_DMAChannel =1;
//  DMA starts   
//TAC5112_20240926	temp = DMA0_CH2_TC_MSW;
temp = DMA1_CH2_TC_MSW;//TAC5112_20240926
	temp |= 0x8000;
//TAC5112_20240926	DMA0_CH2_TC_MSW = temp;
DMA1_CH2_TC_MSW = temp;//TAC5112_20240926

	return 0;
}
//################################################################################
//Uint16 set_dma1_ch3_i2s2_Rin(void)
Uint16 set_dma0_ch3_i2s0_Rin(void)
{
Uint16 temp; 

//TAC5112_20240926	DMA0_CH3_TC_LSW = XMIT_BUFF_SIZE*2;
DMA1_CH3_TC_LSW = XMIT_BUFF_SIZE*2;//TAC5112_20240926

//	DMA0_CH3_TC_MSW = 0x3084;     //src fix, destination increase, sync, auto , int
//TAC5112_20240926	DMA0_CH3_TC_MSW = 0x2084;     //src fix, destination increase,, No auto, int  
DMA1_CH3_TC_MSW = 0x2084;//TAC5112_20240926     //src fix, destination increase,, No auto, int  

//TAC5112_20240926	temp = DMA0_CH32_EVENT_SRC;
temp = DMA1_CH32_EVENT_SRC;//TAC5112_20240926
//TAC5112_20240926	DMA0_CH32_EVENT_SRC = temp | 0x0200;   // I2S2 receive event
DMA1_CH32_EVENT_SRC = temp | 0x0200;//TAC5112_20240926   // I2S2 receive event
    
//TAC5112_20240926	DMA0_CH3_DST_LSW = (Uint16)RxR1_DMA_address;
DMA1_CH3_DST_LSW = (Uint16)RxR1_DMA_address;//TAC5112_20240926
//TAC5112_20240926	DMA0_CH3_DST_MSW = 0xFFFF & (RxR1_DMA_address >> 16);
DMA1_CH3_DST_MSW = 0xFFFF & (RxR1_DMA_address >> 16);//TAC5112_20240926
    
//TAC5112_20240803    DMA0_CH3_SRC_LSW = 0x282C;      // is22 receive right data register lsw
//TAC5112_20240926DMA0_CH3_SRC_LSW = 0x2A2C;      // i2s2 receive right data register lsw
DMA1_CH3_SRC_LSW = 0x2A2C;//TAC5112_20240926      // i2s2 receive right data register lsw

//20140617 T.Okutsu	DMA0_CH3_SRC_LSW = 0x2A2C;      // is22 receive right data register lsw
//TAC5112_20240926	DMA0_CH3_SRC_MSW = 0;
DMA1_CH3_SRC_MSW = 0;//TAC5112_20240926

	//CurrentRxR_DMAChannel =1;
//  DMA starts   
//TAC5112_20240926	temp = DMA0_CH3_TC_MSW;
temp = DMA1_CH3_TC_MSW;//TAC5112_20240926
	temp |= 0x8000;
//TAC5112_20240926	DMA0_CH3_TC_MSW = temp;
DMA1_CH3_TC_MSW = temp;//TAC5112_20240926

	return 0;
}
//################################################################################
void clear_DMA_int(void)
{
Uint16 temp;
	
	//DMA_MSK = 0x00F0;   // 0x1c31 enable DMA1 interrupts
	DMA_MSK = 0x0000;     // Tx-L and Tx-R only
	//DMA_IFR = 0xFFFF;     // 0x1c30 clear interrupt flags
    
//DMA0-CH0 stop
//20200417	temp = DMA1_CH0_TC_MSW;
//TAC5112_20240926	temp = DMA0_CH0_TC_MSW;
temp = DMA1_CH0_TC_MSW;//TAC5112_20240926
	temp &= 0x7FFF;
//20200417	DMA1_CH0_TC_MSW = temp;   
//TAC5112_20240926	DMA0_CH0_TC_MSW = temp;   
DMA1_CH0_TC_MSW = temp;//TAC5112_20240926   
	
//DMA0-CH1 stop
//20200417	temp = DMA1_CH1_TC_MSW;
//TAC5112_20240926	temp = DMA0_CH1_TC_MSW;
temp = DMA1_CH1_TC_MSW;//TAC5112_20240926
	temp &= 0x7FFF;
//20200417	DMA1_CH1_TC_MSW = temp;    
//TAC5112_20240926	DMA0_CH1_TC_MSW = temp;    
DMA1_CH1_TC_MSW = temp;//TAC5112_20240926    
//DMA0-CH2 stop
//20200417	temp = DMA1_CH0_TC_MSW;
//TAC5112_20240926	temp = DMA0_CH2_TC_MSW;
temp = DMA1_CH2_TC_MSW;//TAC5112_20240926
	temp &= 0x7FFF;
//20200417	DMA1_CH0_TC_MSW = temp;   
//TAC5112_20240926	DMA0_CH2_TC_MSW = temp;   
DMA1_CH2_TC_MSW = temp;//TAC5112_20240926   
	
//DMA0-CH3 stop
//20200417	temp = DMA1_CH1_TC_MSW;
//TAC5112_20240926	temp = DMA0_CH3_TC_MSW;
temp = DMA1_CH3_TC_MSW;//TAC5112_20240926
	temp &= 0x7FFF;
//20200417	DMA1_CH1_TC_MSW = temp;    
//TAC5112_20240926	DMA0_CH3_TC_MSW = temp;    
DMA1_CH3_TC_MSW = temp;//TAC5112_20240926    
	
//clear interrupt flags
	//20140617 T.Okutsu	DMA_IFR = 0x0010 | 0x0020;      

//20200417	DMA_IFR = 0x0050 | 0x00A0;      
	DMA_IFR = 0xFFFF;      

}
//****************************************************************************
void enable_dma_int(void)
{
	IER0 |= 0x0100;		// enable DMA Int.Enable bit(CPU Interrupt Enable Reg.)
	DMA_MSK = 0xFFFF;	// enable all interrupts(DMA Interrupt Mask Reg.)
	DMA_IFR = 0xFFFF;	// clear interrupt flags(DMA Interrupt Flag Reg.)
}
//****************************************************************************
void disable_dma_int(void)
{
	IER0 &= ~0x0100;	// disable DMA Int.Enable bit(CPU Interrupt Enable Reg.)
	DMA_MSK = 0;	// disable all interrupts(DMA Interrupt Mask Reg.)
	DMA_IFR = 0xFFFF;	// clear interrupt flags(DMA Interrupt Flag Reg.)
}
//****************************************************************************
void Tx_isr(void)
{
int i;
	for(i=0;i<100;i++){
		
	}
}
//****************************************************************************
// DMAMƂ̃R[obN֐̃vg^CvidvȐ錾Ij
void (*DmaCall_Frnt)(Uint16 dma_chan, Uint16 buffer_no_1, Uint16 buffer_no_2) = (void *)0;

interrupt void DMA_Isr(void)

{
/*20200317
	DMA_IFR = 0xFFFF;	// clear interrupt flags

	set_dma0_ch1_i2s0_Rout();//Ȃ? ch0iLjsRouto͂I
	set_dma0_ch0_i2s0_Lout();//Ȃ? ch0iLjsRouto͂I
20200317*/

/*20200317test*/
//$$$$$$$$$$ 20200317_Appended follows
//20200416	Uint16 temp,i;
Uint16 temp;
Uint16 dma_start;
//Uint32 dma_addr;	//DMA]AhX

	temp = IFR0;
	IFR0 = temp;  

	temp = DMA_IFR;
//    DMA_IFR = temp;     // clear interrupt flags

//dma_addr = (Uint32)XmitL_Sine_1K;
//dma_addr = (dma_addr<<1) + 0x10000;

/*20200318 Only for Sine Wave out test
TxL1_DMA_address = (Uint32)XmitR_Sine_2K;        
TxL1_DMA_address = (TxL1_DMA_address<<1) + 0x10000;	// change word address to byte address and add DARAM offset for DMA
TxL2_DMA_address = (Uint32)XmitR_Sine_2K;        
TxL2_DMA_address = (TxL2_DMA_address<<1) + 0x10000;	// change word address to byte address and add DARAM offset for DMA
20200318 Only for test*/

	    
//o͏
//TAC5112_20240930    if(temp&0x0001)
    if(temp&0x0010)//TAC5112_20240930
//20140616 T.Okutsu	if(temp&0x0010)
	{
		// DMA TxL interrupt (channel 0)
		OutputOperatingForL =1;        
		if(CurrentTxL_DMAChannel ==1)
		{
			CurrentTxL_DMAChannel =2;
//TAC5112_20240926			DMA0_CH0_SRC_LSW = (Uint16)TxL2_DMA_address;
DMA1_CH0_SRC_LSW = (Uint16)TxL2_DMA_address;//TAC5112_20240926
//TAC5112_20240926			DMA0_CH0_SRC_MSW = 0xFFFF & (TxL2_DMA_address >> 16);
DMA1_CH0_SRC_MSW = 0xFFFF & (TxL2_DMA_address >> 16);//TAC5112_20240926
		}
		else
		{
			CurrentTxL_DMAChannel =1;
//TAC5112_20240926			DMA0_CH0_SRC_LSW = (Uint16)TxL1_DMA_address;
DMA1_CH0_SRC_LSW = (Uint16)TxL1_DMA_address;//TAC5112_20240926
//TAC5112_20240926			DMA0_CH0_SRC_MSW = 0xFFFF & (TxL1_DMA_address >> 16);
DMA1_CH0_SRC_MSW = 0xFFFF & (TxL1_DMA_address >> 16);//TAC5112_20240926
		}
		
//TAC5112_20240930		DMA_IFR = 0x0001;     // clear interrupt flags
		DMA_IFR = 0x0010;//TAC5112_20240930     // clear interrupt flags
//20140616 T.Okutsu		DMA_IFR = 0x0010;     // clear interrupt flags
//TAC5112_20240926		dma_start = DMA0_CH0_TC_MSW;
dma_start = DMA1_CH0_TC_MSW;//TAC5112_20240926
	    dma_start |= 0x8000;
//TAC5112_20240926	    DMA0_CH0_TC_MSW = dma_start;
DMA1_CH0_TC_MSW = dma_start;//TAC5112_20240926
	}

//TAC5112_20240926	else if(temp&0x0002)
	else if(temp&0x0020)//TAC5112_20240926
//20140616 T.Okutsu	else if(temp&0x0020)
	{
		// DMA TxR interrupt (channel 1)
		OutputOperatingForR =1;        
		if(CurrentTxR_DMAChannel ==1)
		{
			CurrentTxR_DMAChannel =2;
//TAC5112_20240926			DMA0_CH1_SRC_LSW = (Uint16)TxR2_DMA_address;
DMA1_CH1_SRC_LSW = (Uint16)TxR2_DMA_address;//TAC5112_20240926
//TAC5112_20240926			DMA0_CH1_SRC_MSW = 0xFFFF & (TxR2_DMA_address >> 16);
DMA1_CH1_SRC_MSW = 0xFFFF & (TxR2_DMA_address >> 16);//TAC5112_20240926
		}
		else
		{
			CurrentTxR_DMAChannel =1;
//TAC5112_20240926			DMA0_CH1_SRC_LSW = (Uint16)TxR1_DMA_address;
DMA1_CH1_SRC_LSW = (Uint16)TxR1_DMA_address;//TAC5112_20240926
//TAC5112_20240926			DMA0_CH1_SRC_MSW = 0xFFFF & (TxR1_DMA_address >> 16);
DMA1_CH1_SRC_MSW = 0xFFFF & (TxR1_DMA_address >> 16);//TAC5112_20240926
		}
//TAC5112_20240930		DMA_IFR = 0x0002;     // clear interrupt flags        
		DMA_IFR = 0x0020;//TAC5112_20240930     // clear interrupt flags        
//20140616 T.Okutsu		DMA_IFR = 0x0020;     // clear interrupt flags        
//TAC5112_20240926		dma_start = DMA0_CH1_TC_MSW;
dma_start = DMA1_CH1_TC_MSW;//TAC5112_20240926
		dma_start |= 0x8000;
//TAC5112_20240926		DMA0_CH1_TC_MSW = dma_start;        
DMA1_CH1_TC_MSW = dma_start;//TAC5112_20240926        
	}
	
	//͏
//TAC5112_20240926	else if(temp&0x0004)
	else if(temp&0x0040)//TAC5112_20240926
//20140616 T.Okutsu	else if(temp&0x0040)
	{
		InputOperatingForL =1;        
		// DMA RxL interrupt (channel 2)
		if(CurrentRxL_DMAChannel ==1)
		{
			// change DMA Tx SRC address
			CurrentRxL_DMAChannel =2;
			
//TAC5112_20240926			DMA0_CH2_DST_LSW = (Uint16)RxL2_DMA_address;
DMA1_CH2_DST_LSW = (Uint16)RxL2_DMA_address;//TAC5112_20240926
//TAC5112_20240926			DMA0_CH2_DST_MSW = 0xFFFF & (RxL2_DMA_address >> 16);
DMA1_CH2_DST_MSW = 0xFFFF & (RxL2_DMA_address >> 16);//TAC5112_20240926
		}
		else
		{
			CurrentRxL_DMAChannel =1;
			
//TAC5112_20240926			DMA0_CH2_DST_LSW = (Uint16)RxL1_DMA_address;
DMA1_CH2_DST_LSW = (Uint16)RxL1_DMA_address;//TAC5112_20240926
//TAC5112_20240926			DMA0_CH2_DST_MSW = 0xFFFF & (RxL1_DMA_address >> 16);
DMA1_CH2_DST_MSW = 0xFFFF & (RxL1_DMA_address >> 16);//TAC5112_20240926
		}
//TAC5112_20240930		DMA_IFR = 0x0004;     // clear interrupt flags        
		DMA_IFR = 0x0040;//TAC5112_20240930     // clear interrupt flags        
//20140616 T.Okutsu		DMA_IFR = 0x0040;     // clear interrupt flags        
//TAC5112_20240926		dma_start = DMA0_CH2_TC_MSW;
dma_start = DMA1_CH2_TC_MSW;//TAC5112_20240926
		dma_start |= 0x8000;
//TAC5112_20240926		DMA0_CH2_TC_MSW = dma_start;        
DMA1_CH2_TC_MSW = dma_start;//TAC5112_20240926        
	}

//TAC5112_20240926	else if(temp&0x0008)
	else if(temp&0x0080)//TAC5112_20240926
//20140616 T.Okutsu	else if(temp&0x0080)
	{
		InputOperatingForR =1;                
		// DMA RxR interrupt (channel 3)
		if(CurrentRxR_DMAChannel ==1)
		{
			// change DMA Tx SRC address
			CurrentRxR_DMAChannel =2;
			
//TAC5112_20240926			DMA0_CH3_DST_LSW = (Uint16)RxR2_DMA_address;
DMA1_CH3_DST_LSW = (Uint16)RxR2_DMA_address;//TAC5112_20240926
//TAC5112_20240926			DMA0_CH3_DST_MSW = 0xFFFF & (RxR2_DMA_address >> 16);
DMA1_CH3_DST_MSW = 0xFFFF & (RxR2_DMA_address >> 16);//TAC5112_20240926
		}
		else
		{
			CurrentRxR_DMAChannel =1;
			
//TAC5112_20240926			DMA0_CH3_DST_LSW = (Uint16)RxR1_DMA_address;
DMA1_CH3_DST_LSW = (Uint16)RxR1_DMA_address;//TAC5112_20240926
//TAC5112_20240926			DMA0_CH3_DST_MSW = 0xFFFF & (RxR1_DMA_address >> 16);
DMA1_CH3_DST_MSW = 0xFFFF & (RxR1_DMA_address >> 16);//TAC5112_20240926
		}
//TAC5112_20240930		DMA_IFR = 0x0008;     // clear interrupt flags        
		DMA_IFR = 0x0080;//TAC5112_20240930     // clear interrupt flags        
//20140616 T.Okutsu		DMA_IFR = 0x0080;     // clear interrupt flags        
//TAC5112_20240926		dma_start = DMA0_CH3_TC_MSW;
dma_start = DMA1_CH3_TC_MSW;//TAC5112_20240926
		dma_start |= 0x8000;
//TAC5112_20240926		DMA0_CH3_TC_MSW = dma_start;        
DMA1_CH3_TC_MSW = dma_start;//TAC5112_20240926        
	}
//$$$$$$$$$$ 20200317_Appended above
/*20200317test*/


}
//****************************************************************************
//*                                                                          *
//*  main( )                                                                 *
//*                                                                          *
//****************************************************************************
void main( void )
{
int i, j;
	/* Initialize BSL */
	
Uint8* devdata;//20240925_TAC5112 Test
//20240925_TAC5112 Test Uint16* data;//20240925_TAC5112 Test
	
	//pll_init();
    ConfigPort();
    SYS_GlobalIntEnable();
    PLL_98MHz();

    /* Enable clocks to all peripherals */
    SYS_PCGCR1 = 0x0000;//{[h̪قւ̋CLK̐ݒB, eZDsp5535̏ꍇBSL̒EZDSP5535_init( )B
    SYS_PCGCR2 = 0x0000;//{[h̪قւ̋CLK̐ݒB, eZDsp5535̏ꍇBSL̒EZDSP5535_init( )B
	
	
	
	
//20241004_TAC5112	AIC3204_Init();		//DAC̏ݒB

//====================TAC5112 Read Test==========================
TAC5112_Init();//20240925_TAC5112 Test
// TEST Read TAC5112 Register 20240924
//20241004_TAC5112 	//TAC5112_rset( 1, 1 );//Reset TAC5112 20240924
TAC5112_rset( 1, 0 );//Reset TAC5112 20240924
code_TAC5112 = TAC5112_rget( 0x5 );//Read Reg. = 0x15<-OK! 20240924
//code_TAC5112 = TAC5112_rget( 0x6 );//Read Reg. = 0x55<-OK! 20240924
//code_TAC5112 = TAC5112_rget( 0xA );//Read Reg. = 0x32<-OK! 20240924
//code_TAC5112 = TAC5112_rget( 0x10 );//Read Reg. = 0x52<-OK! 20240924

TAC5112_rset( 0x7F, 0x00 );//Reset TAC5112 20240924
code_TAC5112 = TAC5112_rget( 0x7F );//Read Reg. = 0x15<-OK! 20240924

TAC5112_rset( 0x00, 0x00 );//Reset TAC5112 20240924
code_TAC5112 = TAC5112_rget( 0x00 );//Read Reg. = 0x15<-OK! 20240924

TAC5112_rset( 0x02, 0x09 );//Reset TAC5112 20240927
code_TAC5112 = TAC5112_rget( 0x02 );//Read Reg. = 0x15<-OK! 20240924

TAC5112_rset( 0x3E, 0x50 );//Reset TAC5112 20240927
code_TAC5112 = TAC5112_rget( 0x3E );//Read Reg. = 0x15<-OK! 20240924

TAC5112_rset( 0x40, 0x81 );//Reset TAC5112 20240927
code_TAC5112 = TAC5112_rget( 0x40 );//Read Reg. = 0x15<-OK! 20240924

//TAC5112_rset( 0x4C, 0x5C );//Reset TAC5112 20240927
TAC5112_rset( 0x4C, 0x2E );//Reset TAC5112 20240927
code_TAC5112 = TAC5112_rget( 0x4C );//Read Reg. = 0x15<-OK! 20240924

//TAC5112_rset( 0x50, 0x08 );//Reset TAC5112 20240927
//TAC5112_rset( 0x50, 0x00 );//Reset TAC5112 20240927
//code_TAC5112 = TAC5112_rget( 0x50 );//Read Reg. = 0x15<-OK! 20240924

//TAC5112_rset( 0x76, 0xCC );//Reset TAC5112 20240927
//20241007_TAC5112_Test	TAC5112_rset( 0x76, 0x0C );//Reset TAC5112 20240927
//20241007_TAC5112_Test	code_TAC5112 = TAC5112_rget( 0x76 );//Read Reg. = 0x15<-OK! 20240924

//TAC5112_rset( 0x78, 0xC0 );//Reset TAC5112 20240927
TAC5112_rset( 0x78, 0x40 );//Reset TAC5112 20240927
code_TAC5112 = TAC5112_rget( 0x78 );//Read Reg. = 0x15<-OK! 20240924

TAC5112_rset( 0x79, 0x0C );//Reset TAC5112 20240927
code_TAC5112 = TAC5112_rget( 0x79 );//Read Reg. = 0x15<-OK! 20240924

TAC5112_rset( 0x7A, 0xF0 );//Reset TAC5112 20240927
code_TAC5112 = TAC5112_rget( 0x7A );//Read Reg. = 0x15<-OK! 20240924

TAC5112_rset( 0x1A, 0x40 );//Reset TAC5112 20240927
//TAC5112_rset( 0x1A, 0x41 );// 1slot offset TAC5112 20240927
code_TAC5112 = TAC5112_rget( 0x1A );//Read Reg. = 0x15<-OK! 20240924

	TAC5112_rset( 0x1C, 0x00 );//Reset TAC5112 20240927
	code_TAC5112 = TAC5112_rget( 0x1C );//Read Reg. = 0x15<-OK! 20240924

//20241008_TAC5112	TAC5112_rset( 0x1E, 0x2F );//Reset TAC5112 20240927
//20241008_TAC5112	code_TAC5112 = TAC5112_rget( 0x1E );//Read Reg. = 0x15<-OK! 20240924

//20241007_TAC5112_TesTAC5112_rset( 0x28, 0x21 );//Reset TAC5112 20240927
//20241007_TAC5112_Tescode_TAC5112 = TAC5112_rget( 0x28 );//Read Reg. = 0x15<-OK! 20240924

code_TAC5112 = TAC5112_rget( 0x3E );//Read Reg. = 0x15<-OK! 20240924
code_TAC5112 = TAC5112_rget( 0x40 );//Read Reg. = 0x15<-OK! 20240924

code_TAC5112 = TAC5112_rget( 0x37 );//Read Reg. = 0x15<-OK! 20240924
code_TAC5112 = TAC5112_rget( 0x3C );//Read Reg. = 0x15<-OK! 20240924
code_TAC5112 = TAC5112_rget( 0x3D );//Read Reg. = 0x15<-OK! 20240924


//====================TAC5112 Read Test==========================


	
//20240925_TAC5112 Test	TAC5112_rset( Uint16 regnum, Uint16 regval );//20240925_TAC5112 Test
// TEST Read TAC5112 Register 20240924
	
	
	
	IER0 &= ~0x0020;	// disable _Tx_Isr Int.Enable bit(CPU Interrupt Enable Reg.)//20200416
	

	disable_dma_int();	//DMA݋BiSchɂāj//20200414

//20200317 Appended follows
	CurrentTxL_DMAChannel =2;
	CurrentTxR_DMAChannel =2;
	CurrentRxL_DMAChannel =2;
	CurrentRxR_DMAChannel =2;
	InputOperatingForL =1;
	InputOperatingForR =0;
	OutputOperatingForL =1;
	OutputOperatingForR =0;
	
	setDMA_address();


/*20200414*/
#if(HPF_MODE ==1)
	//ԕϐtB^̒lݒF10Hz @fs=44.1kHz
	//_̏ꍇ̒l
	Q=1.399;
//f=0.0014248;//10Hz @fs=44.1kHz
//20150227	f=0.014248;//TEST fc=100Hz
//20200318	f=0.014248 * 2;//TEST fc=200Hz
f=0.020833 * 2;//TEST fc=100Hz
	Scale=0.9765625;// Scale=32000/32768;
#else
	//ԕϐtB^̒lݒF10Hz @fs=44.1kHz
	//_̏ꍇ̒l
	Q=1.399;
	f=0.0014248;//10Hz @fs=44.1kHz
	//f=0.014248;//TEST fc=100Hz
	Scale=0.9765625;// Scale=32000/32768;
#endif	
	//Œ菬_̏ꍇ̒l
//20200318 	sQ = (short)(Q*32768);
	sQ = (short)(Q*16384);
	sf = (short)(f*32768);
	sScale = (short)(Scale*32768);


	//ϕtB^̒l̐ݒF0.1Hz @fs=(44.1kHz/100=441Hz)
	//fA = 0.0000142;//1 - exp(-2 * M_P * 1/441000);// 0.1 Hz = 1/441000(=0.00000226757)
	fA = 0.00142;//1 - exp(-2 * M_PI * 0.1/44100);// 0.1 Hz = 1/4410(=0.000226757)
	sa =(short)(fA * 32768);

	for(i=0;i<STAGEMAX;i++){
		for(j=0;j<CHANMAX;j++){
			Filter_Out[i][j].High = 0;
			Filter_Out[i][j].Band = 0;
			Filter_Out[i][j].Low = 0;
			
			OnePoleLPF_Output[i][j] = 0;
			OnePoleLPF_OutputxX[i][j] = 0;
			TCount[i][j] = 0;
		}
	}
/*20200414*/

	for(i=0;i<XMIT_BUFF_SIZE;i++){
		RcvL1[i] = 0;
		RcvL2[i] = 0;
		RcvR1[i] = 0;
		RcvR2[i] = 0;
		TrsL1[i] = 0;
		TrsL2[i] = 0;
		TrsR1[i] = 0;
		TrsR2[i] = 0;
		TrsLchBuff_1[i] = 0;
		TrsLchBuff_2[i] = 0;
		TrsRchBuff_1[i] = 0;
		TrsRchBuff_2[i] = 0;
	}

	ReverbInit();//20200319
	

	disable_dma_int();	//DMA݋BiSchɂāj
	clear_DMA_int();//20200417
	USBSTK5505_waitusec( 300 );//20200422

	set_dma0_ch3_i2s0_Rin();
	set_dma0_ch2_i2s0_Lin();
	set_dma0_ch1_i2s0_Rout();
	set_dma0_ch0_i2s0_Lout();

	enable_dma_int();	//DMA݋BiSchɂāj
	
	I2S_Start();
	

	
	while(1)
	{
/*20200430
		//DMA݂hLgꍇ
		//Lch͂ꍇ̏
		if(InputOperatingForL ==1){//DMA0_ch2iLch́jɃf[^ĂhLgꍇAAA
			InputOperatingForL =0;
			if (CurrentRxL_DMAChannel ==2){
//20140617				buff_copy(&RcvL1[0],&TrsL1[0], XMIT_BUFF_SIZE);
//20140617 T.Okutsu				buff_copy(sinetbl, &TrsL1[0], XMIT_BUFF_SIZE);
//20200420				HPFilter(&RcvL1[0],&TrsL1[0], XMIT_BUFF_SIZE, LEFTCH);
				for(i=0;i<XMIT_BUFF_SIZE;i++){
//20200418test	TrsLchBuff_1[i] = RcvL1[i];//20200416test
				}
//20240930_TAC5112	buff_copy(XmitL_Sine_1K, &TrsL1[0], XMIT_BUFF_SIZE);//20240930_TAC5112
//20240930_TAC5112	buff_copy(XmitL_Sine_1K, &TrsR1[0], XMIT_BUFF_SIZE);//20240930_TAC5112
			}
			else{
//20140617				buff_copy(&RcvL2[0],&TrsL2[0], XMIT_BUFF_SIZE);
//20140617 T.Okutsu				buff_copy(sinetbl,&TrsL2[0], XMIT_BUFF_SIZE);
//20200420				HPFilter(&RcvL2[0],&TrsL2[0], XMIT_BUFF_SIZE, LEFTCH);
				for(i=0;i<XMIT_BUFF_SIZE;i++){
//20200418test	TrsLchBuff_2[i] = RcvL2[i];//20200416test
				}
//20240930_TAC5112	buff_copy(XmitL_Sine_1K, &TrsL2[0], XMIT_BUFF_SIZE);//20240930_TAC5112
//20240930_TAC5112	buff_copy(XmitL_Sine_1K, &TrsR2[0], XMIT_BUFF_SIZE);//20240930_TAC5112
			}
		}
20200430*///20240930_TAC5112 
		
		//Rch͂ꍇ̏
		if(InputOperatingForR ==1){//DMA0_ch3iRch́jɃf[^ĂhLgꍇAAA
			InputOperatingForR =0;
			if (CurrentRxR_DMAChannel ==2){
//20140617				buff_copy(&RcvR1[0],&TrsR1[0], XMIT_BUFF_SIZE);
//20140617 T.Okutsu					buff_copy(sinetbl, &TrsR1[0], XMIT_BUFF_SIZE);
//20200420					HPFilter(&RcvR1[0],&TrsR1[0], XMIT_BUFF_SIZE, RIGHTCH);

/*20240930_TAC5112*/
				for(i=0;i<XMIT_BUFF_SIZE;i++){
//20241008_					EchoFx((short)RcvR1[i]);//̪āi32bitjʂްi32bitŁjgMix_L/R,gDelayOut_L/Rɓ͂B
//20240930_TAC5112					TrsL1[i] = gMix_L;
//20240930_TAC5112					TrsR1[i] = gMix_R;
//20241002_TAC5112	TrsL1[i] = 0xA55A;//20241002_TAC5112
//20241002_TAC5112	TrsR1[i] = 0xA55A;//20241002_TAC5112
//20241008_TAC5112	TrsL1[i] = 0x7FFF;//20241002_TAC5112
//20241008_TAC5112	TrsR1[i] = 0x8000;//20241002_TAC5112
//20241009_TAC5112	if(i<=16348){TrsL1[i]=TrsL1[i-1]+340;} else {TrsL1[i]=-16384;}//20241008_TAC5112
//20241009_TAC5112	if(i<=16348){TrsR1[i]=TrsR1[i-1]+340;} else {TrsR1[i]=-16384;}//20241008_TAC5112
	TrsL1[i] = XmitR_Sine_1K[i];//20240930_TAC5112
	TrsR1[i] = XmitR_Sine_1K[i];//20240930_TAC5112
				}
/*20240930_TAC5112*/
//20240930_TAC5112	buff_copy(XmitL_Sine_1K, &TrsL1[0], XMIT_BUFF_SIZE);//20240930_TAC5112
//20240930_TAC5112	buff_copy(XmitL_Sine_1K, &TrsR1[0], XMIT_BUFF_SIZE);//20240930_TAC5112

			}
			else{
//20140617				buff_copy(&RcvR2[0],&TrsR2[0], XMIT_BUFF_SIZE);
//20140617 T.Okutsu					buff_copy(sinetbl, &TrsR2[0], XMIT_BUFF_SIZE);
//20200420					HPFilter(&RcvR2[0],&TrsR2[0], XMIT_BUFF_SIZE, RIGHTCH);

/*20240930_TAC5112*///20240930_TAC5112
				for(i=0;i<XMIT_BUFF_SIZE;i++){
//20241008_					EchoFx((short)RcvR2[i]);//̪āi32bitjʂްi32bitŁjgMix_L/R,gDelayOut_L/Rɓ͂B
//20240930_TAC5112					TrsL2[i] = gMix_L;
//20240930_TAC5112					TrsR2[i] = gMix_R;
//20241002_TAC5112	TrsL2[i] = 0xA55A;//20241002_TAC5112
//20241008_TAC5112	TrsL2[i] = 0x7FFF;//20241002_TAC5112
//20241008_TAC5112	TrsR2[i] = 0x8000;//20241002_TAC5112
//20241002_TAC5112	TrsR2[i] = 0xA55A;//20241002_TAC5112
//20241009_TAC5112	if(i<=16348){TrsL2[i]=TrsL2[i-1]+340;} else {TrsL2[i]=-16384;}//20241008_TAC5112
//20241009_TAC5112	if(i<=16348){TrsR2[i]=TrsR2[i-1]+340;} else {TrsR2[i]=-16384;}//20241008_TAC5112
	TrsL2[i] = XmitR_Sine_1K[i];//20240930_TAC5112
	TrsR2[i] = XmitR_Sine_1K[i];//20240930_TAC5112
				}
/*20240930_TAC5112*///20240930_TAC5112
//20240930_TAC5112	buff_copy(XmitL_Sine_1K, &TrsL2[0], XMIT_BUFF_SIZE);//20240930_TAC5112
//20240930_TAC5112	buff_copy(XmitL_Sine_1K, &TrsR2[0], XMIT_BUFF_SIZE);//20240930_TAC5112

//20240930_TAC5112	for only test 	code_TAC5112 = TAC5112_rget( 0x1A );//Read Reg. = 0x15<-OK! 20241004
			}
		}
	}
}
//****************************************************************************
/*20200416*/
void HPFilter(Int16 *input, Int16 *output, Int16 size, int ch)
{
	Int16 i;
	Int16 ScratchPad;
//	Int32 ScratchPad32;
	
	//HPF(3 Stages) for DSP1
	for(i =0; i<size; i++)
	{
//20140805 LPFeXg̓RgAEgׂ 20200318Test
		ScratchPad = *(input + i);
		StateVariableFiter((short)ScratchPad, ch, 0);
		StateVariableFiter((short)Filter_Out[0][ch].High, ch, 1);
		StateVariableFiter((short)Filter_Out[1][ch].High, ch, 2);
		//*(output + i) = (Int16)Filter_Out[2][ch].Low;
		*(output + i) = (Int16)Filter_Out[2][ch].High;
	}
/*20200419
	//Through Out test
	for(i =0; i<size; i++)
	{
		ScratchPad = *(input + i);
		ScratchPad32 = (long)ScratchPad * 32;
		ScratchPad32 = (long)ScratchPad;
		if(ScratchPad32 > shortUPPERLIMIT) ScratchPad32 = shortUPPERLIMIT;
		else if(ScratchPad32 < shortBOTTOMLIMIT) ScratchPad32 = shortBOTTOMLIMIT;
		*(output + i) = (short)ScratchPad32;
	}
20200419*/
}
//****************************************************************************
void LPFilter(Int16 *input, Int16 *output, Int16 *suboutput, Int16 size, int ch)
{
	Int16 i;
	Int16 ScratchPad;
	Int32 ScratchPad32;
	
	//LPF(3 Stages) for DSP2
	for(i =0; i<size; i++)
	{
		ScratchPad = *(input + i);

//ScratchPad32 = (long)ScratchPad * 8;
//ScratchPad32 = (long)ScratchPad * 2;
		ScratchPad32 = (long)ScratchPad * 64;
		if(ScratchPad32 > shortUPPERLIMIT) ScratchPad32 = shortUPPERLIMIT;
		else if(ScratchPad32 < shortBOTTOMLIMIT) ScratchPad32 = shortBOTTOMLIMIT;
		StateVariableFiter((short)ScratchPad32, ch, 0);
		
		ScratchPad32 = (long)Filter_Out[0][ch].Low;
		if((long)ScratchPad32 > shortUPPERLIMIT)  Filter_Out[0][ch].Low = ScratchPad32 = shortUPPERLIMIT;
		else if((long)ScratchPad32 < shortBOTTOMLIMIT)  Filter_Out[0][ch].Low = ScratchPad32 = shortBOTTOMLIMIT;
		StateVariableFiter((short)ScratchPad32, ch, 1);
		
		ScratchPad32 = (long)Filter_Out[1][ch].Low;
		if((long)ScratchPad32 > shortUPPERLIMIT)  Filter_Out[1][ch].Low = ScratchPad32 = shortUPPERLIMIT;
		else if((long)ScratchPad32 < shortBOTTOMLIMIT)  Filter_Out[1][ch].Low = ScratchPad32 = shortBOTTOMLIMIT;
		StateVariableFiter((short)ScratchPad32, ch, (int)2);
		
//		if(Filter_Out[2][ch].Low > shortUPPERLIMIT) Filter_Out[2][ch].Low = shortUPPERLIMIT;
//		else if(Filter_Out[2][ch].Low < shortBOTTOMLIMIT) Filter_Out[2][ch].Low = shortBOTTOMLIMIT;
//x̑Oȍ~̏ŁA2{ƂɃI[o[t[̌El킹Bibj
if((long)Filter_Out[2][ch].Low > shortUPPERLIMIT/2) Filter_Out[2][ch].Low = shortUPPERLIMIT/2;
else if((long)Filter_Out[2][ch].Low < shortBOTTOMLIMIT/2) Filter_Out[2][ch].Low = shortBOTTOMLIMIT/2;
		
		//Rcho͗pVelocityg`p̐ϕtB^
		if(TCount[0][ch]>=100){
			TCount[0][ch] = 0;//TvOJE^Zbg
		
			//OnePoleLPF(Filter_Out[2][ch].Low * 2, ch, 0);//\p2{ɑătB^́I
			OnePoleLPF(Filter_Out[2][ch].Low, ch, 0);//tB^́I
			
			
		}
		else{
			TCount[0][ch]++;//TvOJE^Zbg
		}
			
		*(output + i) = (Int16)(OnePoleLPF_Output[0][ch] * 32);//\pɑ
		
		//Lcho͗pLPF݂̂̌
		*(suboutput + i) = (Int16)Filter_Out[2][ch].Low * 2;//\p2{ɑI
	}
	

//20140805	//Through Out test
//20140805	for(i =0; i<size; i++)
//20140805	{
//20140805		ScratchPad = *(input + i);
//20140805		ScratchPad32 = (long)ScratchPad * 32;
//20140805		if(ScratchPad32 > shortUPPERLIMIT) ScratchPad32 = shortUPPERLIMIT;
//20140805		else if(ScratchPad32 < shortBOTTOMLIMIT) ScratchPad32 = shortBOTTOMLIMIT;
//20140805		*(output + i) = (short)ScratchPad32;
//20140805	}

}
//****************************************************************************
void StateVariableFiter(short Input, int ch, int stage)
{

	//begin loop
	Filter_Out[stage][ch].Low = (short)((long)Filter_Out[stage][ch].Low + ((long)((long)sf * (long)Filter_Out[stage][ch].Band)>>15));
    if((long)Filter_Out[stage][ch].Low > shortUPPERLIMIT) Filter_Out[stage][ch].Low = shortUPPERLIMIT;
    else if((long)Filter_Out[stage][ch].Low < shortBOTTOMLIMIT) Filter_Out[stage][ch].Low = shortBOTTOMLIMIT;

	Filter_Out[stage][ch].High = (short)(((long)((long)sScale * (long)Input)>>15) - (long)Filter_Out[stage][ch].Low - ((long)((long)sQ * (long)Filter_Out[stage][ch].Band)>>15));
    if((long)Filter_Out[stage][ch].High > shortUPPERLIMIT) Filter_Out[stage][ch].High = shortUPPERLIMIT;
    else if((long)Filter_Out[stage][ch].High < shortBOTTOMLIMIT) Filter_Out[stage][ch].High = shortBOTTOMLIMIT;

	Filter_Out[stage][ch].Band = (short)(((long)((long)sf * (long)Filter_Out[stage][ch].High)>>15) + (long)Filter_Out[stage][ch].Band);
    if((long)Filter_Out[stage][ch].Band > shortUPPERLIMIT) Filter_Out[stage][ch].Band = shortUPPERLIMIT;
    else if((long)Filter_Out[stage][ch].Band < shortBOTTOMLIMIT) Filter_Out[stage][ch].Band = shortBOTTOMLIMIT;

//20140805	//Notch
//20140805	Filter_Out[stage][ch].Notch = (short)((short)Filter_Out[stage][ch].High + (short)Filter_Out[stage][ch].Low);
//20140805    if(Filter_Out[stage][ch].Notch > shortUPPERLIMIT) Filter_Out[stage][ch].Notch = shortUPPERLIMIT;
//20140805    else if(Filter_Out[stage][ch].Notch < shortBOTTOMLIMIT) Filter_Out[stage][ch].Notch = shortBOTTOMLIMIT;
//20140805	//End loop

}
//****************************************************************************
void OnePoleLPF(short Input, int ch, int stage)
{
	OnePoleLPF_Output[stage][ch] += (short)((((long)Input - (long)OnePoleLPF_Output[stage][ch]) * sa)>>15);
	if((long)OnePoleLPF_Output[stage][ch] > shortUPPERLIMIT){
		//OnePoleLPF_Output[stage][ch] = shortUPPERLIMIT;
		OnePoleLPF_Output[stage][ch] /= 2;
	}
	else if((long)OnePoleLPF_Output[stage][ch] < shortBOTTOMLIMIT){
		//OnePoleLPF_Output[stage][ch] = shortBOTTOMLIMIT;
		OnePoleLPF_Output[stage][ch] /= 2;
	}
}
/*20200416*/
//****************************************************************************
void ReverbInit(void)
{
	//GR[̐ݒ菉l
	gFxSendVal = 0x3FFF;
	gFBVal_L = gFBVal_R = 0x1FFF;
	gXFBVal_L = gXFBVal_R = 0x1FFF;

	gFxParamNum = 3;
	
	DrySft[0] = 127;
	DrySft[1] = 127;
	DrySft[2] = 127;
	DrySft[3] = 32;
	DrySft[4] = 95;


//1st Delayline(L/R)	
	gFxParam1[0].gDelayLength_L = 1;//No Echo
	gFxParam1[0].gDelayLength_R = 1;//No Echo
	gFxParam1[1].gDelayLength_L = 5500;
	gFxParam1[1].gDelayLength_R = 3200;
	gFxParam1[2].gDelayLength_L = 1800;
	gFxParam1[2].gDelayLength_R = 1080;
	gFxParam1[3].gDelayLength_L = 2300;
	gFxParam1[3].gDelayLength_R = 1500;
	gFxParam1[4].gDelayLength_L = 5250;
	gFxParam1[4].gDelayLength_R = 6000;

	gFxParam1[0].FxSendSft = 0;
	gFxParam1[1].FxSendSft = 64;
	gFxParam1[2].FxSendSft = 82;
//20200422	gFxParam1[3].FxSendSft = 80;
	gFxParam1[3].FxSendSft = 80;
	gFxParam1[4].FxSendSft = 92;
	
	gFxParam1[0].FB_LSft = 32;	
	gFxParam1[1].FB_LSft = 48;	
	gFxParam1[2].FB_LSft = 23;	
	gFxParam1[3].FB_LSft = 0;	
	gFxParam1[4].FB_LSft = 21;	
	
	gFxParam1[0].FB_RSft = 32;	
	gFxParam1[1].FB_RSft = 48;	
	gFxParam1[2].FB_RSft = 23;	
	gFxParam1[3].FB_RSft = 0;	
	gFxParam1[4].FB_RSft = 21;	
	
	gFxParam1[0].XFB_LSft = 32;	
	gFxParam1[1].XFB_LSft = 40;	
	gFxParam1[2].XFB_LSft = 23;	
	gFxParam1[3].XFB_LSft = 7;	
	gFxParam1[4].XFB_LSft = 21;	

	gFxParam1[0].XFB_RSft = 32;	
	gFxParam1[1].XFB_RSft = 40;	
	gFxParam1[2].XFB_RSft = 23;	
	gFxParam1[3].XFB_RSft = 7;	
	gFxParam1[4].XFB_RSft = 21;	


//2nd Delayline(L/R)
	gFxParam2[0].gDelayLength_L = 1;//No Echo
	gFxParam2[0].gDelayLength_R = 1;//No Echo
	gFxParam2[1].gDelayLength_L = 1;
	gFxParam2[1].gDelayLength_R = 1;
	gFxParam2[2].gDelayLength_L = 6000;
	gFxParam2[2].gDelayLength_R = 6000;
//20200626	gFxParam2[3].gDelayLength_L = 3600;
//20200626	gFxParam2[3].gDelayLength_R = 4300;
	gFxParam2[3].gDelayLength_L = 5300;
	gFxParam2[3].gDelayLength_R = 4500;
	gFxParam2[4].gDelayLength_L = 6000;
	gFxParam2[4].gDelayLength_R = 6000;
	
	gFxParam2[0].FB_LSft = 0;	
	gFxParam2[1].FB_LSft = 0;	
	gFxParam2[2].FB_LSft = 30;	
	gFxParam2[3].FB_LSft = 0;	
	gFxParam2[4].FB_LSft = 21;	
	
	gFxParam2[0].FB_RSft = 0;	
	gFxParam2[1].FB_RSft = 0;	
	gFxParam2[2].FB_RSft = 24;	
	gFxParam2[3].FB_RSft = 0;	
	gFxParam2[4].FB_RSft = 21;	
	
	gFxParam2[0].XFB_LSft = 0;	
	gFxParam2[1].XFB_LSft = 0;	
	gFxParam2[2].XFB_LSft = 22;	
//20200422	gFxParam2[3].XFB_LSft = 22;	
	gFxParam2[3].XFB_LSft = 15;	
	gFxParam2[4].XFB_LSft = 22;	

	gFxParam2[0].XFB_RSft = 0;	
	gFxParam2[1].XFB_RSft = 0;	
	gFxParam2[2].XFB_RSft = 22;	
//20200422	gFxParam2[3].XFB_RSft = 22;	
	gFxParam2[3].XFB_RSft = 15;	
	gFxParam2[4].XFB_RSft = 22;	
	
	gFxParam3[0].gDelayLength_L = 1;//No Echo
	gFxParam3[0].gDelayLength_R = 1;//No Echo
	gFxParam3[1].gDelayLength_L = 1;
	gFxParam3[1].gDelayLength_R = 1;
	gFxParam3[2].gDelayLength_L = 6000;
	gFxParam3[2].gDelayLength_R = 6000;
	gFxParam3[3].gDelayLength_L = 5000;
	gFxParam3[3].gDelayLength_R = 6000;
	gFxParam3[4].gDelayLength_L = 6000;
	gFxParam3[4].gDelayLength_R = 6000;
	
	gFxParam3[0].FB_LSft = 0;	
	gFxParam3[1].FB_LSft = 0;	
	gFxParam3[2].FB_LSft = 30;	
	gFxParam3[3].FB_LSft = 0;	
	gFxParam3[4].FB_LSft = 21;	
	
	gFxParam3[0].FB_RSft = 0;	
	gFxParam3[1].FB_RSft = 0;	
	gFxParam3[2].FB_RSft = 25;	
	gFxParam3[3].FB_RSft = 0;	
	gFxParam3[4].FB_RSft = 21;	
	
	gFxParam3[0].XFB_LSft = 0;	
	gFxParam3[1].XFB_LSft = 0;	
	gFxParam3[2].XFB_LSft = 24;	
//20200422	gFxParam3[3].XFB_LSft = 65;	
//20200624	gFxParam3[3].XFB_LSft = 70;	
	gFxParam3[3].XFB_LSft = 92;	
	gFxParam3[4].XFB_LSft = 22;	

	gFxParam3[0].XFB_RSft = 0;	
	gFxParam3[1].XFB_RSft = 0;	
	gFxParam3[2].XFB_RSft = 24;	
//20200422	gFxParam3[3].XFB_RSft = 65;	
//20200624	gFxParam3[3].XFB_RSft = 70;	
	gFxParam3[3].XFB_RSft = 92;	
	gFxParam3[4].XFB_RSft = 22;	

//20200415
	gCurrentADRS = 0;
	gOD_CurrentADRS = 0;
	gChrs_CurrentADRS = 0;
	PutDataToDelayLine(gDelayIn_L, gDelayIn_R);
	PutDataToOD_DelayLine(gOD_DelayIn);
	PutDataToChrs_DelayLine(gChrs_DelayIn);
	SetDataToDelayOut();	
//20200415

}
//******************************************************************************
void PutDataToDelayLine(short LchData, short RchData)
{//L/Rިڲײ݂̍ŌiCurrentDRSjDataςݏグB(for̓)TvɂÅ֐s邱ƁI

	gDelayLine_Lch[gCurrentADRS] = LchData;
	gDelayLine_Rch[gCurrentADRS] = RchData;

//ifɂLq*
	//۰ٕϐ"gCurrentADRS"̍XV(i̊֐Ă΂xɍsj
	if(gCurrentADRS < DELAYLINE_MAX-1){//ޯ̧eʔ͈͓ł΁AAA
		gCurrentADRS++;
	}
	else{//ޯ̧eʂςł΁AAA
		gCurrentADRS = 0;
	}
}
//******************************************************************************
void PutDataToOD_DelayLine(short Data)
{//L/Rިڲײ݂̍ŌiCurrentDRSjDataςݏグB(for̓)TvɂÅ֐s邱ƁI

	gOD_DelayLine[gOD_CurrentADRS] = Data;

//ifɂLq*
	//۰ٕϐ"gCurrentADRS"̍XV(i̊֐Ă΂xɍsj
	if(gOD_CurrentADRS < OD_DELAYLINE_MAX-1){//ޯ̧eʔ͈͓ł΁AAA
		gOD_CurrentADRS++;
	}
	else{//ޯ̧eʂςł΁AAA
		gOD_CurrentADRS = 0;
	}
}
//******************************************************************************
void PutDataToChrs_DelayLine(short Data)
{//L/Rިڲײ݂̍ŌiCurrentDRSjDataςݏグB(for̓)TvɂÅ֐s邱ƁI

	gChrs_DelayLine[gChrs_CurrentADRS] = Data;

//ifɂLq*
	//۰ٕϐ"gCurrentADRS"̍XV(i̊֐Ă΂xɍsj
	if(gChrs_CurrentADRS < CHRS_DELAYLINE_MAX-1){//ޯ̧eʔ͈͓ł΁AAA
		gChrs_CurrentADRS++;
	}
	else{//ޯ̧eʂςł΁AAA
		gChrs_CurrentADRS = 0;
	}
}
//******************************************************************************
void SetDataToDelayOut(void)//ݓ_gCurrentADRSɑ΂DelayLengthߋްlԂB
{
short DelayADRS_R, DelayADRS_Tail_a, DelayADRS_Frnt_a, Chrs_DelayADRS_1 = 0;
short DelayADRS_L, DelayADRS_Tail_b, DelayADRS_Frnt_b, Chrs_DelayADRS_2 = 0;


	//ȉAOctave Downp̏
	// an̏
	//DelaySpaň㔼͈(Tail= Frntŋ߂̐M)̏iɏAɃNXtF[hCj
	if(gOD_CurrentADRS > HalfDelayPoint_Tail_a){
		DelayADRS_Tail_a = gOD_CurrentADRS -  HalfDelayPoint_Tail_a;
	//
	// ########## "V"̉ӏKvDelayǏoPoint ++++vZ****ɔfI ##########
	//    |<--HalfDelayPoint_Tail->|          |HalfDelaySpan
	//    0   V<--HalfDelayPoint_Tail->|CurrentADRS                           |OD_DELAYLINE_MAX
	//    |***|--------------------|+++|------|-------------------------------|
	//    <-ߋް<----------------|<------------------ Ɖߋް<--
	//                               
	}
	else{
		DelayADRS_Tail_a = OD_DELAYLINE_MAX-1 - (HalfDelayPoint_Tail_a - gOD_CurrentADRS);//
	//
	//    |<----HalfDelayPoint_Tail---->|
	//    0         CurrentADRS|++++++++|     HalfDelaySpan          V        |OD_DELAYLINE_MAX
	//    |---------------------------------|------------------------|********|
	//    <-ߋް<--------|<-------------------------- Ɖߋް<--
	//                       
	}
	
	//DelaySpan̑O͈(Frnt=Ȃߋ̐M)̏iɑ傫AɃNXtF[hAEgj
	if(gOD_CurrentADRS > HalfDelayPoint_Frnt_a){
		DelayADRS_Frnt_a = gOD_CurrentADRS - HalfDelayPoint_Frnt_a;
	//
	// ########## "V"̉ӏKvDelayǏoPoint ++++vZ****ɔfI ##########
	//    |<----------------HalfDelayPoint_Frnt----------------->|++++++|CurrentADRS
	//    0      V                           HalfDelaySpan              |     |OD_DELAYLINE_MAX
	//    |******|--------------------------|---------------------------|-----|
	//    <-ߋް<-------------------------------------------------|<----- Ɖߋް
	//                                                                
	}
	else{
		DelayADRS_Frnt_a = OD_DELAYLINE_MAX-1 - (HalfDelayPoint_Frnt_a - gOD_CurrentADRS);//
	//
	//    |<---------------------HalfDelayPoint_Frnt----------------------->|
	//    |                                                 |CurrentADRS    |
	//    0                                  HalfDelaySpan  |+V+++++++++++++| |OD_DELAYLINE_MAX
	//    |---------------------------------|-----------------|***************|
	//    <-ߋް<-------------------------------------|<----------------- Ɖߋް
	//                                                    
	}
	
	// bn̏
	//DelaySpaň㔼͈(Tail= Frntŋ߂̐M)̏iɏAɃNXtF[hCj
	if(gOD_CurrentADRS > HalfDelayPoint_Tail_b){
		DelayADRS_Tail_b = gOD_CurrentADRS -  HalfDelayPoint_Tail_b;
	}
	else{
		DelayADRS_Tail_b = OD_DELAYLINE_MAX-1 - (HalfDelayPoint_Tail_b - gOD_CurrentADRS);//
	}
	
	//DelaySpan̑O͈(Frnt=Ȃߋ̐M)̏iɑ傫AɃNXtF[hAEgj
	if(gOD_CurrentADRS > HalfDelayPoint_Frnt_b){
		DelayADRS_Frnt_b = gOD_CurrentADRS - HalfDelayPoint_Frnt_b;
	}
	else{
		DelayADRS_Frnt_b = OD_DELAYLINE_MAX-1 - (HalfDelayPoint_Frnt_b - gOD_CurrentADRS);//
	}

	//gOctDwnOut_b, gOctDwnOut_aɏo͂ۂɂ́AHalfDelaySpan̒ɉNXtF[hI
	gOctDwnOut_a = (short)((((long)gOD_DelayLine[DelayADRS_Frnt_a] * (HalfDelaySpan_a - XFadeLevel_a)) + ((long)gOD_DelayLine[DelayADRS_Tail_a] * (XFadeLevel_a))) / HalfDelaySpan_a);//
	gOctDwnOut_b = (short)((((long)gOD_DelayLine[DelayADRS_Frnt_b] * (HalfDelaySpan_b - XFadeLevel_b)) + ((long)gOD_DelayLine[DelayADRS_Tail_b] * (XFadeLevel_b))) / HalfDelaySpan_b);//

	//fBC|CgisIifBC|CgXɌ̉AǏosb`1/2Bj
	if(HalfDelayCount_a < (HalfDelaySpan_a - 1)){
		HalfDelayCount_a++;
	}
	else{
		HalfDelayCount_a = 0;
		XFadeLevel_a = 0;
	}
	HalfDelayPoint_Tail_a = HalfDelayCount_a/2;//ɒPɃAhX1/2({͂Q_̃jAԂׂ)
	HalfDelayPoint_Frnt_a = HalfDelaySpan_a/2 + HalfDelayPoint_Tail_a;
	
	//NXtF[hEx̉Z(ł́AHalfDealySpanƓlƂ)
	if(XFadeLevel_a < HalfDelaySpan_a){
		XFadeLevel_a++;
	}

	//fBC|CgisIifBC|CgXɌ̉AǏosb`1/2Bj
	if(HalfDelayCount_b < (HalfDelaySpan_b - 1)){
		HalfDelayCount_b++;
	}
	else{
		HalfDelayCount_b = 0;
		XFadeLevel_b = 0;
	}
	HalfDelayPoint_Tail_b = HalfDelayCount_b/2;//ɒPɃAhX1/2({͂Q_̃jAԂׂ)
	HalfDelayPoint_Frnt_b = HalfDelaySpan_b/2 + HalfDelayPoint_Tail_b;
	
	//NXtF[hEx̉Z(ł́AHalfDealySpanƓlƂ)
	if(XFadeLevel_b < HalfDelaySpan_b){
		XFadeLevel_b++;
	}
	//ȏAOctave Downp̏


//20210405	//ȉAChorus
//20210405	if(gChrs_CurrentADRS > (CHRS_DELAYLINE_MAX-1)){
//20210405		Chrs_DelayADRS = gChrs_CurrentADRS - CHRS_DELAYLINE_MAX-1;
//20210405	}
//20210405	else{
//20210405		Chrs_DelayADRS = CHRS_DELAYLINE_MAX-1 - (CHRS_DELAYLINE_MAX-1- gChrs_CurrentADRS);//
//20210405	}

	//ȉAChorus
	LFO(1);
	gChrs_DelayLength_1 = (short)(512 + (int)LFOValue * 7/1024);
	gChrs_DelayLength_2 = (short)(512 - (int)LFOValue * 7/1024);

	if(gChrs_CurrentADRS > gChrs_DelayLength_1){
		Chrs_DelayADRS_1 = gChrs_CurrentADRS - gChrs_DelayLength_1;
	}
	else{
		Chrs_DelayADRS_1 = CHRS_DELAYLINE_MAX-1 - (gChrs_DelayLength_1- gChrs_CurrentADRS);//
	}
	if(gChrs_CurrentADRS > gChrs_DelayLength_2){
		Chrs_DelayADRS_2 = gChrs_CurrentADRS - gChrs_DelayLength_2;
	}
	else{
		Chrs_DelayADRS_2 = CHRS_DELAYLINE_MAX-1 - (gChrs_DelayLength_2- gChrs_CurrentADRS);//
	}
	
	gChrs_DelayOut_1 = (short)gChrs_DelayLine[Chrs_DelayADRS_1];
	gChrs_DelayOut_2 = (short)gChrs_DelayLine[Chrs_DelayADRS_2];//Chorusʕt



	//ȉAReverbp̏
	//Lch1
	if(gCurrentADRS > gFxParam1[gFxParamNum].gDelayLength_L){
		DelayADRS_L = gCurrentADRS - gFxParam1[gFxParamNum].gDelayLength_L;
	//
	// ########## "V"̉ӏKvDelayPoint ++++vZ****ɔfI ##########
	//
	//    |<--------DelayLength------------>|      CurrentADRS
	//    0      V<--------DelayLength------------>|                                      |DELAYLINE_MAX
	//    |******|--------------------------|++++++|--------------------------------------|
	//    <-ߋް  <------------------------------------------ Ɖߋް<--
	//
	}
	else{
		DelayADRS_L = DELAYLINE_MAX-1 - (gFxParam1[gFxParamNum].gDelayLength_L- gCurrentADRS);//
	//
	// ########## "V"̉ӏKvDelayPoint ++++vZ****ɔfI ##########
	//
	//    |<--------DelayLength------------>|
	//    0              CurrentADRS        |                          V                  |DELAYLINE_MAX
	//    |--------------|++++++++++++++++++|--------------------------|******************|
	//    <-ߋް  <------------------------------------------ Ɖߋް<--
	//
	}
	gDelayOut_L1 = ((short)gDelayLine_Lch[DelayADRS_L]);//
	
	//Rch1
	if(gCurrentADRS > gFxParam1[gFxParamNum].gDelayLength_R){
		DelayADRS_R = gCurrentADRS - gFxParam1[gFxParamNum].gDelayLength_R;
	}
	else{
		DelayADRS_R = DELAYLINE_MAX-1 - (gFxParam1[gFxParamNum].gDelayLength_R- gCurrentADRS);//
	}
	gDelayOut_R1 = ((short)gDelayLine_Rch[DelayADRS_R]);//
	

	//Lch2
	if(gCurrentADRS > gFxParam2[gFxParamNum].gDelayLength_L){
		DelayADRS_L = gCurrentADRS - gFxParam2[gFxParamNum].gDelayLength_L;
	}
	else{
		DelayADRS_L = DELAYLINE_MAX-1 - (gFxParam2[gFxParamNum].gDelayLength_L- gCurrentADRS);//
	}
	gDelayOut_L2 = ((short)gDelayLine_Lch[DelayADRS_L]);//
	
	//Rch2
	if(gCurrentADRS > gFxParam2[gFxParamNum].gDelayLength_R){
		DelayADRS_R = gCurrentADRS - gFxParam2[gFxParamNum].gDelayLength_R;
	}
	else{
		DelayADRS_R = DELAYLINE_MAX-1 - (gFxParam2[gFxParamNum].gDelayLength_R- gCurrentADRS);//
	}
	gDelayOut_R2 = ((short)gDelayLine_Rch[DelayADRS_R]);//
	
	//Lch3
	if(gCurrentADRS > gFxParam3[gFxParamNum].gDelayLength_L){
		DelayADRS_L = gCurrentADRS - gFxParam3[gFxParamNum].gDelayLength_L;
	}
	else{
		DelayADRS_L = DELAYLINE_MAX-1 - (gFxParam3[gFxParamNum].gDelayLength_L- gCurrentADRS);//
	}
	gDelayOut_L3 = ((short)gDelayLine_Lch[DelayADRS_L]);//
	
	//Rch3
	if(gCurrentADRS > gFxParam3[gFxParamNum].gDelayLength_R){
		DelayADRS_R = gCurrentADRS - gFxParam3[gFxParamNum].gDelayLength_R;
	}
	else{
		DelayADRS_R = DELAYLINE_MAX-1 - (gFxParam3[gFxParamNum].gDelayLength_R- gCurrentADRS);//
	}
	gDelayOut_R3 = ((short)gDelayLine_Rch[DelayADRS_R]);//
	
	//ȏAReverbp̏
	
}
//******************************************************************************
short LFO(short Turm)
{
	if(LFODir==1){
		LFOValue += Turm;
		if(LFOValue > LFOVALUEMAX){
			LFODir = 0;
			LFOValue = LFOVALUEMAX;
		}
	}
	else{
		LFOValue -= Turm;
		if(LFOValue < 0){
			LFODir = 1;
			LFOValue = 0;
		}
	}
}
//******************************************************************************
//20210402	void EchoFx(short gDirectSrc)//̪āi32bitjʂްi32bitŁjgMix_L/R,gDelayOut_L/Rɓ͂B
void EchoFx(short Src)//̪āi32bitjʂްi32bitŁjgMix_L/R,gDelayOut_L/Rɓ͂B
{
//20200416	gOctDwnOut_a = gOctDwnOut_b = gDelayOut_R2 = gDelayOut_L2 = gDelayOut_R3 = gDelayOut_L3 = 0;//20200416test

	//
	//̪Čn֑قZoB
//20210402	gFxSend = (short)(((long)gDirectSrc * gFxParam1[gFxParamNum].FxSendSft)/128);
//20210405	gFxSend = (short)(((long)Src * gFxParam1[gFxParamNum].FxSendSft)/128);
	gFxSend = (short)(((long)(Src + gChrs_DelayOut_1 + gChrs_DelayOut_2)/2 * gFxParam1[gFxParamNum].FxSendSft)/128);
	
//20200410	gFB_L = /*(short)(((long)gOctDwnOut_b * gFxParam1[gFxParamNum].FB_LSft)/128) +*/ (short)(((long)gDelayOut_L2 * gFxParam2[gFxParamNum].FB_LSft)/128) + (short)(((long)gDelayOut_L3 * gFxParam3[gFxParamNum].FB_LSft)/128);//DeepReso
//20200410	gFB_R = /*(short)(((long)gOctDwnOut_a * gFxParam1[gFxParamNum].FB_RSft)/128) +*/ (short)(((long)gDelayOut_R2 * gFxParam2[gFxParamNum].FB_RSft)/128) + (short)(((long)gDelayOut_R3 * gFxParam3[gFxParamNum].FB_RSft)/128);//DeepReso
	gFB_L = (short)(((long)gDelayOut_L1 * gFxParam1[gFxParamNum].FB_LSft)/128) + (short)(((long)gDelayOut_L2 * gFxParam2[gFxParamNum].FB_LSft)/128) + (short)(((long)gDelayOut_L3 * gFxParam3[gFxParamNum].FB_LSft)/128);//DeepReso
	gFB_R = (short)(((long)gDelayOut_R1 * gFxParam1[gFxParamNum].FB_RSft)/128) + (short)(((long)gDelayOut_R2 * gFxParam2[gFxParamNum].FB_RSft)/128) + (short)(((long)gDelayOut_R3 * gFxParam3[gFxParamNum].FB_RSft)/128);//DeepReso
	
	//L/Ř݂n̨ޯقZoB
//20200410	XgFB_L = /*(short)(((long)gOctDwnOut_a * gFxParam1[gFxParamNum].XFB_LSft)/128) +*/ (short)(((long)gDelayOut_R2 * gFxParam2[gFxParamNum].XFB_LSft)/128) + (short)(((long)gDelayOut_R3 * gFxParam3[gFxParamNum].XFB_LSft)/128);
//20200410	XgFB_R = /*(short)(((long)gOctDwnOut_b * gFxParam1[gFxParamNum].XFB_RSft)/128) +*/ (short)(((long)gDelayOut_L2 * gFxParam2[gFxParamNum].XFB_RSft)/128) + (short)(((long)gDelayOut_L3 * gFxParam3[gFxParamNum].XFB_RSft)/128);
	XgFB_L = (short)(((long)gDelayOut_L1 * gFxParam1[gFxParamNum].XFB_LSft)/128) + (short)(((long)gDelayOut_R2 * gFxParam2[gFxParamNum].XFB_LSft)/128) + (short)(((long)gDelayOut_R3 * gFxParam3[gFxParamNum].XFB_LSft)/128);
	XgFB_R = (short)(((long)gDelayOut_R1 * gFxParam1[gFxParamNum].XFB_RSft)/128) + (short)(((long)gDelayOut_L2 * gFxParam2[gFxParamNum].XFB_RSft)/128) + (short)(((long)gDelayOut_L3 * gFxParam3[gFxParamNum].XFB_RSft)/128);

	//ިڲײ݂ɑMЯB
//20200406	gDelayIn_L = (short)(/*gFB_L + XgFB_L +*/ gFxSend);//L/Rch̨ޯƴ̪ľނZB
//20200406	gDelayIn_R = (short)(/*gFB_R + XgFB_R +*/ gFxSend);//L/Rch̨ޯƴ̪ľނZB
	gDelayIn_L = (short)(gOctDwnOut_b/2 + gOctDwnOut_a/2 - gFB_L + XgFB_L + gFxSend);//L/Rch̨ޯƴ̪ľނZB
	gDelayIn_R = (short)(gOctDwnOut_b/2 + gOctDwnOut_a/2 - gFB_R + XgFB_R + gFxSend);//L/Rch̨ޯƴ̪ľނZB

	//ŏIo͂̐MЯB
//20200408	gMix_L = (short)( ((long)gDirectSrc * DrySft[gFxParamNum]/128) + ((((long)gOctDwnOut_b + (long)gDelayOut_L2 / 2 + (long)gDelayOut_L3) / 2 * (256 - DrySft[gFxParamNum]))/128) );
//20200408	gMix_R = (short)( ((long)gDirectSrc * DrySft[gFxParamNum]/128) + ((((long)gOctDwnOut_a + (long)gDelayOut_R2 / 2 + (long)gDelayOut_R3) / 2 * (256 - DrySft[gFxParamNum]))/128) );
//20210402	gMix_L = (short)( ((long)gDirectSrc * DrySft[gFxParamNum]/128) + ((((long)gOctDwnOut_b * 2 + ((long)gDelayOut_L1 / 4 + (long)gDelayOut_L2 / 4 + (long)gDelayOut_L3 / 4)  - gDelayIn_L) * (180 - DrySft[gFxParamNum]))/128) );
//20210402	gMix_R = (short)( ((long)gDirectSrc * DrySft[gFxParamNum]/128) + ((((long)gOctDwnOut_a * 2 + ((long)gDelayOut_R1 / 4 + (long)gDelayOut_R2 / 4 + (long)gDelayOut_R3 / 4)  - gDelayIn_R) * (180 - DrySft[gFxParamNum]))/128) );
//20210405	gMix_L = (short)( ((long)Src * DrySft[gFxParamNum]/128) + ((((long)gOctDwnOut_b * 2 + ((long)gDelayOut_L1 / 4 + (long)gDelayOut_L2 / 4 + (long)gDelayOut_L3 / 4)  - gDelayIn_L) * (180 - DrySft[gFxParamNum]))/128) );
//20210405	gMix_R = (short)( ((long)Src * DrySft[gFxParamNum]/128) + ((((long)gOctDwnOut_a * 2 + ((long)gDelayOut_R1 / 4 + (long)gDelayOut_R2 / 4 + (long)gDelayOut_R3 / 4)  - gDelayIn_R) * (180 - DrySft[gFxParamNum]))/128) );
//20210406	gMix_L = (short)(((long)(Src+gChrs_DelayOut_1+gChrs_DelayOut_2)/2 * DrySft[gFxParamNum]/128) + ((((long)gOctDwnOut_b * 2 + ((long)gDelayOut_L1/4 + (long)gDelayOut_L2/4 + (long)gDelayOut_L3/4) - gDelayIn_L) * (180 - DrySft[gFxParamNum]))/128) );
//20210406	gMix_R = (short)(((long)(Src+gChrs_DelayOut_1+gChrs_DelayOut_2)/2 * DrySft[gFxParamNum]/128) + ((((long)gOctDwnOut_a * 2 + ((long)gDelayOut_R1/4 + (long)gDelayOut_R2/4 + (long)gDelayOut_R3/4) - gDelayIn_R) * (180 - DrySft[gFxParamNum]))/128) );
	gMix_L = (short)(((long)(gChrs_DelayOut_1+gChrs_DelayOut_2)/1 * DrySft[gFxParamNum]/128) + ((((long)gOctDwnOut_b * 2 + ((long)gDelayOut_L1/4 + (long)gDelayOut_L2/4 - (long)gDelayOut_L3/4)) * (128 - DrySft[gFxParamNum]))/128) );
	gMix_R = (short)(((long)(gChrs_DelayOut_1+gChrs_DelayOut_2)/1 * DrySft[gFxParamNum]/128) + ((((long)gOctDwnOut_a * 2 + ((long)gDelayOut_R1/4 + (long)gDelayOut_R2/4 - (long)gDelayOut_R3/4)) * (128 - DrySft[gFxParamNum]))/128) );

//20210406	gMix_L = gChrs_DelayOut_1;//20210405
//20210406	gMix_R = gChrs_DelayOut_2;//20210405

	//ިڲײݏ̕KvȒx_̏o͂oBިڲײ݂֐MuB
	SetDataToDelayOut();	//gDelayOut_LgDelayOut_RꊇĂIij
	PutDataToDelayLine((short)gDelayIn_L, (short)gDelayIn_R);//EƂɊeXްިڲײ݂ɓ́B
//20210406	PutDataToOD_DelayLine((short)Src + gOctDwnOut_b/4 + gOctDwnOut_a/4);////OctDown͐\[XOctDown x Octdown x ...̏񏈗IB
	PutDataToOD_DelayLine((short)Src);//OctDown͐\[X̂(OctDown x Octdown x ...̏񏈗͂ȂI)B
//20210405	PutDataToChrs_DelayLine((short)Src);//Chorus͐\[X݂̂ɂI
	PutDataToChrs_DelayLine((short)Src + gOctDwnOut_b/2 + gOctDwnOut_a/2);//OctDownɂChorusI

}
//******************************************************************************
void buff_copy(Int16 *input, Int16 *output, Int16 size)
{
	Int16 i;
	
	for(i =0; i<size; i++)
	{
		*(output + i) = *(input +i);
	}
	
}
//****************************************************************************

//********************************* EOF **************************************
